@@ -23,7 +23,7 @@ worst=0
 Small = {
     #"_v_indexprops" : IndexProps(auto=1),
     # var1 column will be indexed if not heavy test
-    "var1" : StringCol(length=4, dflt="", indexed=1, pos=2),
+    "var1" : StringCol(length=4, dflt="Hi!", indexed=1, pos=2),
     "var2" : IntCol(0, indexed=1, pos=1),
     "var3" : FloatCol(0, indexed=1, pos=0),
     #"var4" : BoolCol(0, indexed=1),
@@ -79,7 +79,7 @@ def createNewBenchFile(bfile, verbose):
                 for atom in ["string", "int", "float"]:
                     bf.createTable(groupM, atom, Search, atom+" bench")
     bf.close()
-    
+
 def createFile(filename, nrows, filters, index, heavy, auto, verbose):
 
     # Open a file in "w"rite mode
@@ -94,8 +94,8 @@ def createFile(filename, nrows, filters, index, heavy, auto, verbose):
         Small["_v_indexprops"] = IndexProps(auto=0, filters=filters)
     if heavy:
         # make the string entry not indexed
-        Small["var1"] = StringCol(length=4, dflt="", indexed=0)
-        
+        Small["var1"] = StringCol(length=4, dflt="Hi!", indexed=0)
+
     # Create the test table
     table = fileh.createTable(fileh.root, 'table', Small, "test table",
                               None, nrows)
@@ -119,7 +119,10 @@ def createFile(filename, nrows, filters, index, heavy, auto, verbose):
         if not heavy:
             for n in xrange(j-i):
                 var1[n] = str("%.4s" % var2[n])
-        table.append([var3, var2, var1])
+#           var1[:] = str(var2)[:4]
+        else:
+            var1[:] = "Hi !"
+        table.append([var3*var3, var2, var1])
     table.flush()
     rowswritten += nrows
     time1 = time.time()-t1
@@ -145,7 +148,7 @@ def createFile(filename, nrows, filters, index, heavy, auto, verbose):
         if not auto:
             print "Time for indexing:", round(time2,3), \
                   "iKrows/s:", round(indexrows/1000./time2,3),
-	tcpu2 = time.clock()-cpu1
+        tcpu2 = time.clock()-cpu1
     else:
         indexrows = 0
         time2 = 0.0000000001  # an ugly hack
@@ -159,7 +162,7 @@ def createFile(filename, nrows, filters, index, heavy, auto, verbose):
             print "NOT indexing rows"
     # Close the file
     fileh.close()
-        
+
     size2 = os.stat(filename)[6] - size1
     if not auto and index:
         print ", Index size:", round(size2/(1024.*1024.), 3), "MB"
@@ -176,7 +179,7 @@ def benchCreate(file, nrows, filters, index, bfile, heavy, auto,
         table = bf.getNode("/"+recsize+"/create_worst")
     else:
         table = bf.getNode("/"+recsize+"/create_best")
-        
+
     (rowsw, irows, rowsz, time1, time2, tcpu1, tcpu2, size1, size2) = \
             createFile(file, nrows, filters, index, heavy, auto, verbose)
     # Collect data
@@ -212,7 +215,7 @@ def benchCreate(file, nrows, filters, index, bfile, heavy, auto,
     table.row["rowseci"] = rowseci
     table.row.append()
     bf.close()
-    
+
 def readFile(filename, atom, riter, indexmode, verbose):
     # Open the HDF5 file in read-only mode
 
@@ -247,7 +250,7 @@ def readFile(filename, atom, riter, indexmode, verbose):
     print "Select mode:", indexmode, ". Selecting for type:", atom
     # The interval for look values at. This is aproximately equivalent to
     # the number of elements to select
-    chunksize = 1000  # Change here for selecting more or less entries 
+    chunksize = 1000  # Change here for selecting more or less entries
     # Initialize the random generator always with the same integer
     # in order to have reproductible results
     random.seed(19)
@@ -258,18 +261,18 @@ def readFile(filename, atom, riter, indexmode, verbose):
         t1 = time.time()
         if atom == "string":
             if indexmode in ["indexed", "inkernel"]:
-                results = [p.nrow()
+                results = [p.nrow
                            # for p in where("1000" <= var1 <= "1010")]
                            #for p in where(var1 == "1111")]
                            for p in where(var1 == str(rnd)[-4:])]
             else:
-                results = [p.nrow() for p in table
+                results = [p.nrow for p in table
                            # if "1000" <= p["var1"] <= "1010"]
                            #if p["var1"] == "1111"]
                            if p["var1"] == str(rnd)[-4:]]
         elif atom == "int":
             if indexmode in ["indexed", "inkernel"]:
-                results = [p.nrow()
+                results = [p.nrow
                            # for p in where(2+i<= var2 < 10+i)]
                            # for p in where(2<= var2 < 10)]
                            # for p in where(110*i <= var2 < 110*(i+1))]
@@ -278,7 +281,7 @@ def readFile(filename, atom, riter, indexmode, verbose):
                            #for p in where(rnd <= var2 < rnd+3)]
                            for p in where(rnd <= var2 < rnd+chunksize)]
             else:
-                results = [p.nrow() for p in table
+                results = [p.nrow for p in table
                            # if p["var2"] < 10+i]
                            # if 2 <= p["var2"] < 10)]
                            # if 110*i <= p["var2"] < 110*(i+1)]
@@ -289,7 +292,7 @@ def readFile(filename, atom, riter, indexmode, verbose):
         elif atom == "float":
             if indexmode in ["indexed", "inkernel"]:
                 t1=time.time()
-                results = [p.nrow()
+                results = [p.nrow
                            # for p in where(var3 < 5.)]
                            #for p in where(3. <= var3 < 5.)]
                            #for p in where(float(rnd) <= var3 < float(rnd+3))]
@@ -300,7 +303,7 @@ def readFile(filename, atom, riter, indexmode, verbose):
                 #print "time for complete selection-->", time.time()-t1
                 #print "results-->", results, rnd
             else:
-                results = [p.nrow() for p in table
+                results = [p.nrow for p in table
                            # if p["var3"] < 5.]
                            #if 3. <= p["var3"] < 5.]
                            #if float(rnd) <= p["var3"] < float(rnd+3)]
@@ -325,7 +328,7 @@ def readFile(filename, atom, riter, indexmode, verbose):
             else:
                 time2 += time.time() - t1
                 tcpu2 += time.clock() - cpu1
-                        
+
     if riter > 1:
         if indexmode == "indexed" and riter >= 5:
             correction = 5
@@ -339,8 +342,8 @@ def readFile(filename, atom, riter, indexmode, verbose):
 
     #rowsread = table.nrows * riter
     rowsread = table.nrows
-    rowsize = table.rowsize 
-        
+    rowsize = table.rowsize
+
     # Close the file
     fileh.close()
 
@@ -355,7 +358,7 @@ def benchSearch(file, riter, indexmode, bfile, heavy, psyco, verbose):
         tableparent = "/"+recsize+"/search_worst/"+indexmode+"/"
     else:
         tableparent = "/"+recsize+"/search_best/"+indexmode+"/"
-        
+
     # Do the benchmarks
     if not heavy:
         #atomlist = ["string", "int", "float", "bool"]
@@ -419,16 +422,16 @@ if __name__=="__main__":
         psyco_imported = 1
     except:
         psyco_imported = 0
-    
+
     import time
-    
+
     usage = """usage: %s [-v] [-p] [-a] [-r] [-w] [-c level] [-l complib] [-S] [-F] [-n nrows] [-x] [-b file] [-t] [-h] [-k riter] [-m indexmode] datafile
             -v verbose
-	    -p use "psyco" if available
+            -p use "psyco" if available
             -a automatic indexing (default is separate indexing)
             -R use Random values for filling
-	    -r only read test
-	    -w only write test
+            -r only read test
+            -w only write test
             -c sets a compression level (do not set it or 0 for no compression)
             -l sets the compression library ("zlib", "lzo", "ucl", "bzip2" or "none")
             -S activate shuffling filter
@@ -448,7 +451,7 @@ if __name__=="__main__":
         sys.exit(0)
 
     # if we pass too much parameters, abort
-    if len(pargs) <> 1: 
+    if len(pargs) <> 1:
         sys.stderr.write(usage)
         sys.exit(0)
 
@@ -518,7 +521,7 @@ if __name__=="__main__":
         # This means no compression at all
         complib="zlib"  # just to make PyTables not complaining
         complevel=0
-        
+
     # Catch the hdf5 file passed as the last argument
     file = pargs[0]
 
