@@ -7,25 +7,58 @@
 #include "H5Zbzip2.h"  		       /* Import FILTER_BZIP2 */
 
 
-/*-------------------------------------------------------------------------
- *
- * Private functions
- * These are a replica of those in H5LT.c, but get_attribute_string_sys
- * needs them, so it is better to copy them here.
- * F. Altet 2004-04-20
- *
- *-------------------------------------------------------------------------
- */
-
-herr_t _open_id( hid_t loc_id,
-		 const char *obj_name,
-		 int obj_type );
-
-herr_t _close_id( hid_t obj_id,
-		  int obj_type );
-
 /* ---------------------------------------------------------------- */
 
+#ifdef WIN32
+#include <windows.h>
+
+/* This routine is meant to detect whether a dynamic library can be
+   loaded on Windows. This is only way to detect its presence without
+   harming the user.
+*/
+int getLibrary(char *libname) {
+    HINSTANCE hinstLib;
+
+    /* Load the dynamic library */
+    hinstLib = LoadLibrary(TEXT(libname));
+
+    if (hinstLib != NULL) {
+      /* Free the dynamic library */
+      FreeLibrary(hinstLib);
+      return 0;
+    }
+    else {
+      return -1;
+    }
+}
+
+#else  /* Unix platforms */
+#include <dlfcn.h>
+
+/* Routine to detect the existance of shared libraries in UNIX. This
+   has to be checked in MacOSX. However, this is not used right now in
+   utilsExtension.pyx because UNIX does not complain when trying to
+   load an extension library that depends on a shared library that it
+   is not in the system (python raises just the ImportError). */
+int getLibrary(char *libname) {
+    void *hinstLib;
+
+    /* Load the dynamic library */
+    hinstLib = dlopen(libname, RTLD_LAZY);
+
+    if (hinstLib != NULL) {
+      /* Free the dynamic library */
+      dlclose(hinstLib);
+      return 0;
+    }
+    else {
+      return -1;
+    }
+}
+
+
+#endif  /* Win32 */
+
 herr_t set_cache_size(hid_t file_id, size_t cache_size) {
   herr_t code;
 
@@ -38,8 +71,8 @@ herr_t set_cache_size(hid_t file_id, size_t cache_size) {
   code = H5Fget_mdc_config(file_id, &config);
   config.set_initial_size = TRUE;
   config.initial_size = cache_size;
-  config.incr_mode = H5C_incr__off;
-  config.decr_mode = H5C_decr__off;
+/*   config.incr_mode = H5C_incr__off; */
+/*   config.decr_mode = H5C_decr__off; */
 /*   printf("Setting cache size to: %d\n", cache_size); */
   code = H5Fset_mdc_config(file_id, &config);
 /*   printf("Return code for H5Fset_mdc_config: %d\n", code); */
@@ -54,24 +87,6 @@ PyObject *_getTablesVersion() {
   return PyString_FromString(PYTABLES_VERSION);
 }
 
-/* PyObject *getZLIBVersionInfo(void) { */
-/*   long binver; */
-/*   PyObject *t; */
-
-/* #ifdef ZLIB_VERNUM		/\* Only available for zlib >= 1.2 *\/ */
-/*   binver = ZLIB_VERNUM;  	/\* This is not exactly the user's lib */
-/* 				   version but that of the binary */
-/* 				   packager version!  However, this */
-/* 				   should be not too important *\/ */
-/* #else */
-/*   binver = 1;  			/\* For version of zlib < 1.2 *\/ */
-/* #endif */
-/*   t = PyTuple_New(2); */
-/*   PyTuple_SetItem(t, 0, PyInt_FromLong(binver)); */
-/*   PyTuple_SetItem(t, 1, PyString_FromString(zlibVersion())); */
-/*   return t; */
-/* } */
-
 PyObject *getHDF5VersionInfo(void) {
   long binver;
   unsigned majnum, minnum, relnum;
@@ -108,10 +123,14 @@ PyObject *createNamesTuple(char *buffer[], int nelements)
 {
   int i;
   PyObject *t;
+  PyObject *str;
 
   t = PyTuple_New(nelements);
   for (i = 0; i < nelements; i++) {
-    PyTuple_SetItem(t, i, PyString_FromString(buffer[i]) );
+    str = PyString_FromString(buffer[i]);
+    PyTuple_SetItem(t, i, str);
+    /* PyTuple_SetItem does not need a decref, because it already do this */
+/*     Py_DECREF(str); */
   }
   return t;
 }
@@ -120,10 +139,14 @@ PyObject *createNamesList(char *buffer[], int nelements)
 {
   int i;
   PyObject *t;
+  PyObject *str;
 
   t = PyList_New(nelements);
   for (i = 0; i < nelements; i++) {
-    PyList_SetItem(t, i, PyString_FromString(buffer[i]) );
+    str = PyString_FromString(buffer[i]);
+    PyList_SetItem(t, i, str);
+    /* PyList_SetItem does not need a decref, because it already do this */
+/*     Py_DECREF(str); */
   }
   return t;
 }
@@ -178,7 +201,7 @@ PyObject *get_filter_names( hid_t loc_id,
      for (i=0; i<nf; i++) {
        cd_nelmts = 20;
 #if H5_VERS_MAJOR == 1 && H5_VERS_MINOR < 7
-       /* 1.6.2 */
+       /* 1.6.x */
        filt_id = H5Pget_filter(dcpl, i, &filt_flags, &cd_nelmts,
 			       cd_values, sizeof(f_name), f_name);
 #else
@@ -219,7 +242,8 @@ out:
 **
 ****************************************************************/
 herr_t gitercb(hid_t loc_id, const char *name, void *data) {
-  PyObject **out_info=(PyObject **)data;
+  PyObject   **out_info=(PyObject **)data;
+  PyObject   *strname;
   herr_t     ret;            /* Generic return value         */
   H5G_stat_t statbuf;
 
@@ -229,12 +253,14 @@ herr_t gitercb(hid_t loc_id, const char *name, void *data) {
     ret = H5Gget_objinfo(loc_id, name, FALSE, &statbuf);
 /*     CHECK(ret, FAIL, "H5Gget_objinfo"); */
 
+    strname = PyString_FromString(name);
     if (statbuf.type == H5G_GROUP) {
-      PyList_Append(out_info[0], PyString_FromString(name));
+      PyList_Append(out_info[0], strname);
     }
     else if (statbuf.type == H5G_DATASET) {
-      PyList_Append(out_info[1], PyString_FromString(name));
+      PyList_Append(out_info[1], strname);
     }
+    Py_DECREF(strname);
 
     return(0);  /* Loop until no more objects remain in directory */
 }
@@ -269,9 +295,12 @@ PyObject *Giterate(hid_t parent_id, hid_t loc_id, const char *name) {
 **
 ****************************************************************/
 static herr_t aitercb( hid_t loc_id, const char *name, void *op_data) {
+  PyObject *strname;
 
+  strname = PyString_FromString(name);
   /* Return the name of the attribute on op_data */
-  PyList_Append(op_data, PyString_FromString(name));
+  PyList_Append(op_data, strname);
+  Py_DECREF(strname);
   return(0);    /* Loop until no more attrs remain in object */
 }
 
@@ -301,34 +330,31 @@ PyObject *Aiterate(hid_t loc_id) {
 H5T_class_t getHDF5ClassID(hid_t loc_id,
 			   const char *name,
 			   H5D_layout_t *layout,
-			   hid_t *type_id) {
-   hid_t        dataset_id;
+			   hid_t *type_id,
+			   hid_t *dataset_id) {
    H5T_class_t  class_id;
    hid_t        plist;
 
    /* Open the dataset. */
-   if ( (dataset_id = H5Dopen( loc_id, name )) < 0 )
+   if ( (*dataset_id = H5Dopen( loc_id, name )) < 0 )
      return -1;
 
    /* Get an identifier for the datatype. */
-   *type_id = H5Dget_type( dataset_id );
+   *type_id = H5Dget_type( *dataset_id );
 
    /* Get the class. */
    class_id = H5Tget_class( *type_id );
 
    /* Get the layout of the datatype */
-   plist = H5Dget_create_plist(dataset_id);
+   plist = H5Dget_create_plist(*dataset_id);
    *layout = H5Pget_layout(plist);
    H5Pclose(plist);
 
-   /* End access to the dataset */
-   if ( H5Dclose( dataset_id ) )
-     return -1;
-
    return class_id;
 
 }
 
+
 /* Helper routine that returns the rank, dims and byteorder for
    UnImplemented objects. 2004
 */
@@ -466,7 +492,7 @@ hsize_t _PyEval_SliceIndex_modif(PyObject *v, hsize_t *pi)
 }
 
 /* This has been copied from the Python 2.3 sources in order to get a
-   funtion similar to the method slice.indices(length) but that works
+   function similar to the method slice.indices(length) but that works
    with 64-bit ints and not only with ints.
  */
 
@@ -532,185 +558,6 @@ hsize_t getIndicesExt(PyObject *s, hsize_t length,
 }
 
 
-/*-------------------------------------------------------------------------
- * Function: get_attribute_string_sys
- *
- * Purpose: Reads a attribute specific of PyTables in a fast way
- *
- * Return: Success: 0, Failure: -1
- *
- * Programmer: Francesc Altet, faltet@carabos.com
- *
- * Date: September 19, 2003
- *
- * Comments:
- *
- * Modifications:
- *
- *-------------------------------------------------------------------------
- */
-
-
-PyObject *get_attribute_string_sys( hid_t loc_id,
-				    const char *obj_name,
-				    const char *attr_name)
-{
-
- /* identifiers */
- hid_t      obj_id;
- hid_t      attr_id;
- hid_t      attr_type;
- size_t     attr_size;
- PyObject   *attr_value;
- char       *data;
- H5G_stat_t statbuf;
-
- /* Get the type of object */
- if (H5Gget_objinfo(loc_id, obj_name, 1, &statbuf)<0)
-  return NULL;
-
- /* Open the object */
- if ((obj_id = _open_id( loc_id, obj_name, statbuf.type )) < 0)
-   return NULL;
-
-/*  Check if attribute exists */
- /* This is commented out to make the attribute reading faster */
-/*  if (H5LT_find_attribute(obj_id, attr_name) <= 0)  */
- if ( ( attr_id = H5Aopen_name( obj_id, attr_name ) ) < 0 )
-   /* If the attribute does not exists, return None */
-   /* and do not even warn the user */
-   return Py_None;
-
- if ( (attr_type = H5Aget_type( attr_id )) < 0 )
-  goto out;
-
- /* Get the size. */
- attr_size = H5Tget_size( attr_type );
-
-/*  printf("name: %s. size: %d\n", attr_name, attr_size); */
- /* Allocate memory for the input buffer */
- data = (unsigned char *)malloc(attr_size);
-
- if ( H5Aread( attr_id, attr_type, data ) < 0 )
-  goto out;
-
- attr_value = PyString_FromString(data);
- free(data);
-
- if ( H5Tclose( attr_type )  < 0 )
-  goto out;
-
- if ( H5Aclose( attr_id ) < 0 )
-  return Py_None;
-
- /* Close the object */
- if ( _close_id( obj_id, statbuf.type ) < 0 )
-  return Py_None;
-
- return attr_value;
-
-out:
- H5Aclose( attr_id );
- H5Aclose( attr_type );
- return Py_None;
-
-}
-
-/*-------------------------------------------------------------------------
- * Function: _open_id
- *
- * Purpose: Private function used by get_attribute_string_sys
- *
- * Return: Success: 0, Failure: -1
- *
- * Programmer: Pedro Vicente, pvn@ncsa.uiuc.edu
- *
- * Date: September 19, 2002
- *
- * Comments:
- *
- *-------------------------------------------------------------------------
- */
-
-
-
-herr_t _open_id( hid_t loc_id,
-		 const char *obj_name,
-		 int obj_type /*basic object type*/ )
-{
-
- hid_t   obj_id = -1;
-
- switch ( obj_type )
- {
-  case H5G_DATASET:
-
-   /* Open the dataset. */
-   if ( (obj_id = H5Dopen( loc_id, obj_name )) < 0 )
-    return -1;
-   break;
-
-  case H5G_GROUP:
-
-   /* Open the group. */
-   if ( (obj_id = H5Gopen( loc_id, obj_name )) < 0 )
-    return -1;
-   break;
-
-  default:
-   return -1;
- }
-
- return obj_id;
-
-}
-
-
-/*-------------------------------------------------------------------------
- * Function: _close_id
- *
- * Purpose: Private function used by get_attribute_string_sys
- *
- * Return: Success: 0, Failure: -1
- *
- * Programmer: Pedro Vicente, pvn@ncsa.uiuc.edu
- *
- * Date: September 19, 2002
- *
- * Comments:
- *
- *-------------------------------------------------------------------------
- */
-
-
-
-herr_t _close_id( hid_t obj_id,
-		  int obj_type /*basic object type*/ )
-{
-
- switch ( obj_type )
- {
-  case H5G_DATASET:
-   /* Close the dataset. */
-   if ( H5Dclose( obj_id ) < 0 )
-    return -1;
-   break;
-
-  case H5G_GROUP:
-  /* Close the group. */
-   if ( H5Gclose( obj_id ) < 0 )
-    return -1;
-   break;
-
-  default:
-   return -1;
- }
-
- return 0;
-
-}
-
-
 /* The next provides functions to support a complex datatype.
    HDF5 does not provide an atomic type class for complex numbers
    so we make one from a HDF5 compound type class.
@@ -753,7 +600,7 @@ static H5T_order_t get_complex_order(hid_t type_id) {
    instead of a nested type should not be a big issue (I hope!) :-/
    F. Altet 2005-05-23 */
 int is_complex(hid_t type_id) {
-  hid_t class_id, base_type_id, base_class_id;
+  hid_t class_id, base_type_id;
   hid_t class1, class2;
   char *colname1, *colname2;
   int result = 0;
