@@ -4,8 +4,7 @@
 #       Created: October 10, 2002
 #       Author:  Francesc Altet - faltet@carabos.com
 #
-#       $Source: /home/ivan/_/programari/pytables/svn/cvs/pytables/pytables/tables/Array.py,v $
-#       $Id: Array.py 1181 2005-09-12 17:49:13Z faltet $
+#       $Id: Array.py 1675 2006-06-19 13:16:21Z faltet $
 #
 ########################################################################
 
@@ -39,13 +38,19 @@ try:
 except ImportError:
     Numeric_imported = False
 
+try:
+    import numpy
+    numpy_imported = True
+except ImportError:
+    numpy_imported = False
+
 import tables.hdf5Extension as hdf5Extension
 from tables.utils import calcBufferSize, processRange, processRangeRead, \
-                         convToFlavor, CHUNKTIMES
+                         convToFlavor, is_idx
 from tables.Leaf import Leaf, Filters
 
 
-__version__ = "$Revision: 1181 $"
+__version__ = "$Revision: 1675 $"
 
 
 # default version for ARRAY objects
@@ -55,6 +60,10 @@ __version__ = "$Revision: 1181 $"
 #obversion = "2.2"    # This adds support for time datatypes.
 obversion = "2.3"    # This adds support for enumerated datatypes.
 
+byteorderDict = {'>':'big',
+                 '<':'little',
+                 '=':sys.byteorder,
+                 '|':'non-relevant'}
 
 
 class Array(hdf5Extension.Array, Leaf):
@@ -63,8 +72,7 @@ class Array(hdf5Extension.Array, Leaf):
     It enables to create new datasets on-disk from Numeric, numarray,
     lists, tuples, strings or scalars, or open existing ones.
 
-    All Numeric and numarray typecodes are supported except for complex
-    datatypes.
+    All Numeric and numarray typecodes are supported.
 
     Methods:
 
@@ -73,13 +81,12 @@ class Array(hdf5Extension.Array, Leaf):
 
     Instance variables:
 
-        type -- The type class for the array.
-        itemsize -- The size of the atomic items. Specially useful for
-            CharArray objects.
-        flavor -- The object type of this object ("NumArray", "CharArray",
-            "Numeric", "List", "Tuple", "String", "Int" or "Float").
-        nrows -- The value of the first dimension of Array.
+        flavor -- The object representation of this array.
+        nrows -- The length of the first dimension of the array.
         nrow -- On iterators, this is the index of the current row.
+        type -- The type class of the represented array.
+        stype -- The string type of the represented array.
+        itemsize -- The size of the base items.
 
     """
 
@@ -141,50 +148,130 @@ class Array(hdf5Extension.Array, Leaf):
 
         return buffersize
 
-    def __init__(self, object = None, title = ""):
+
+    def __init__(self, parentNode, name,
+                 object=None, title="",
+                 log=True):
         """Create the instance Array.
 
         Keyword arguments:
 
         object -- The (regular) object to be saved. It can be any of
-            NumArray, CharArray, Numeric, List, Tuple, String, Int of
-            Float types, provided that they are regular (i.e. they are
-            not like [[1,2],2]).
+            numarray, numpy, numeric, list, tuple, string, integer of
+            floating point types, provided that they are regular
+            (i.e. they are not like [[1,2],2]).
 
         title -- Sets a TITLE attribute on the HDF5 array entity.
-
         """
-        self._v_new_title = title
-        # Assign some filter values by default, i.e. no filters for Array
-        self._v_new_filters = Filters()
-        self.extdim = -1   # An Array object is not enlargeable
-        # Check if we have to create a new object or read their contents
-        # from disk
-        if object is not None:
-            self._v_new = 1
-            self.object = object
-        else:
-            self._v_new = 0
 
-    def _create(self):
+        self._v_version = None
+        """The object version of this array."""
+
+        self._v_new = new = object is not None
+        """Is this the first time the node has been created?"""
+        self._v_new_title = title
+        """New title for this node."""
+
+        self._object = object
+        """
+        The object to be stored in the array.  It can be any of
+        ``numarray``, ``numpy``, ``numeric``, list, tuple, string,
+        integer of floating point types, provided that they are
+        regular (i.e. they are not like ``[[1, 2], 2]``).
+        """
+        self._v_expectedrows = None
+        """The expected number of rows to be stored in the array."""
+        self.byteorder = None
+        """
+        The endianness of data in memory ('big', 'little' or
+        'non-relevant').
+        """
+        self.rowsize = None
+        """The size in bytes of each row in the array."""
+        self._v_maxTuples = None
+        """The maximum number of rows that are read on each chunk iterator."""
+        self._v_chunksize = None
+        """
+        The HDF5 chunk size for ``CArray``, ``EArray`` and ``VLArray``
+        objects.
+        """
+        self._v_convert = True
+        """Whether the ``Array`` object must be converted or not."""
+        self.shape = None
+        """The shape of the stored array."""
+        self._enum = None
+        """The enumerated type containing the values in this array."""
+
+        # Miscellaneous iteration rubbish.
+        self._start = None
+        """Starting row for the current iteration."""
+        self._stop = None
+        """Stopping row for the current iteration."""
+        self._step = None
+        """Step size for the current iteration."""
+        self._nrowsread = None
+        """Number of rows read up to the current state of iteration."""
+        self._startb = None
+        """Starting row for current buffer."""
+        self._stopb = None
+        """Stopping row for current buffer. """
+        self._row = None
+        """Current row in iterators (sentinel)."""
+        self._init = False
+        """Whether we are in the middle of an iteration or not (sentinel)."""
+        self.listarr = None
+        """Current buffer in iterators."""
+
+        # Documented (*public*) attributes.
+        self.flavor = None
+        """The object representation of this array.  It can be any of
+        'numarray', 'numpy', 'numeric' or 'python' values."""
+        self.nrows = None
+        """The length of the first dimension of the array."""
+        self.nrow = None
+        """On iterators, this is the index of the current row."""
+        self.type = None
+        """The type class of the represented array."""
+        self.stype = None
+        """The string type of the represented array."""
+        self.itemsize = None
+        """
+        The size of the base items (specially useful for ``CharType``
+        objects).
+        """
+        self.extdim = -1   # ordinary arrays are not enlargeable
+        """The index of the enlargeable dimension."""
+
+        # Ordinary arrays have no filters: leaf is created with default ones.
+        super(Array, self).__init__(parentNode, name, new, Filters(), log)
+
+
+    def _g_create(self):
         """Save a fresh array (i.e., not present on HDF5 file)."""
-        global obversion
 
         self._v_version = obversion
         try:
-            naarr, self.flavor = self._convertIntoNA(self.object)
+            naarr, self.flavor = self._convertIntoNA(self._object)
         except:  #XXX
             # Problems converting data. Close the node and re-raise exception.
-            #print "Problems converting input object:", str(self.object)
+            #print "Problems converting input object:", str(self._object)
             self.close(flush=0)
             raise
 
+        # Decrease the number of references to the object
+        self._object = None
+
         if naarr.shape:
             self._v_expectedrows = naarr.shape[0]
         else:
             self._v_expectedrows = 1  # Scalar case
         if (isinstance(naarr, strings.CharArray)):
             self.byteorder = "non-relevant"
+            if naarr.itemsize() == 0:
+                # Null strings are not supported. Close node and re-raise.
+                self.close(flush=0)
+                raise NotImplementedError, \
+                      "Empty strings are not supported yet. Sorry about that!"
         else:
             self.byteorder  = naarr._byteorder
 
@@ -201,117 +288,32 @@ class Array(hdf5Extension.Array, Leaf):
         (self._v_maxTuples, self._v_chunksize) = \
                             calcBufferSize(self.rowsize, self._v_expectedrows)
 
-        self.shape = naarr.shape
-        if naarr.shape:
+        # If shape has not been assigned yet, assign it.
+        if self.shape is None:
+            self.shape = naarr.shape
+        if self.shape != ():
             self.nrows = naarr.shape[0]
         else:
             self.nrows = 1    # Scalar case
         self.itemsize = naarr.itemsize()
         try:
-            self.type, self.stype = self._createArray(naarr, self._v_new_title)
+            (self._v_objectID, self.type, self.stype) = (
+                self._createArray(naarr, self._v_new_title))
+            return self._v_objectID
         except:  #XXX
             # Problems creating the Array on disk. Close node and re-raise.
             self.close(flush=0)
             raise
 
-    def _convertIntoNA(self, object):
-        "Convert a generic object into a numarray object"
-        arr = object
-        if isinstance(arr, numarray.NumArray):
-            flavor = "NumArray"
-            naarr = arr
-            self.byteorder  = naarr._byteorder
-        elif isinstance(arr, strings.CharArray):
-            flavor = "CharArray"
-            naarr = arr
-            self.byteorder = "non-relevant"
-        elif (Numeric_imported and type(arr) == type(Numeric.array(1))):
-            flavor = "Numeric"
-            if arr.typecode() == "c":
-                # To emulate as close as possible Numeric character arrays,
-                # itemsize for chararrays will be always 1
-                if arr.iscontiguous():
-                    # This the fastest way to convert from Numeric to numarray
-                    # because no data copy is involved
-                    naarr = strings.array(buffer(arr),
-                                          itemsize=1,
-                                          shape=arr.shape)
-                else:
-                    # Here we absolutely need a copy so as to obtain a buffer.
-                    # Perhaps this can be avoided or optimized by using
-                    # the tolist() method, but this should be tested.
-                    naarr = strings.array(buffer(arr.copy()),
-                                          itemsize=1,
-                                          shape=arr.shape)
-            else:
-                if arr.iscontiguous():
-                    # This the fastest way to convert from Numeric to numarray
-                    # because no data copy is involved
-                    naarr = numarray.array(buffer(arr),
-                                           type=arr.typecode(),
-                                           shape=arr.shape)
-                else:
-                    # Here we absolutely need a copy in order
-                    # to obtain a buffer.
-                    # Perhaps this can be avoided or optimized by using
-                    # the tolist() method, but this should be tested.
-                    naarr = numarray.array(buffer(arr.copy()),
-                                           type=arr.typecode(),
-                                           shape=arr.shape)
-
-        elif (isinstance(arr, tuple) or
-              isinstance(arr, list)):
-            # Test if can convert to numarray object
-            try:
-                naarr = numarray.array(arr)
-            # If not, test with a chararray
-            except TypeError:
-                try:
-                    naarr = strings.array(arr)
-                # If still doesn't, issues an error
-                except:  #XXX
-                    raise TypeError, \
-"""The object '%s' can't be converted into a numerical or character array.
-Sorry, but this object is not supported.""" % (arr)
-            if isinstance(arr, tuple):
-                flavor = "Tuple"
-            else:
-                flavor = "List"
-        elif isinstance(arr, int):
-            naarr = numarray.array(arr)
-            flavor = "Int"
-        elif isinstance(arr, float):
-            naarr = numarray.array(arr)
-            flavor = "Float"
-        elif isinstance(arr, str):
-            naarr = strings.array(arr)
-            flavor = "String"
-        else:
-            raise TypeError, \
-"""The object '%s' is not in the list of supported objects (NumArray, CharArray, Numeric, homogeneous list or homogeneous tuple, int, float or str). Sorry, but this object is not supported.""" % (arr)
-
-        # We always want a contiguous buffer
-        # (no matter if has an offset or not; that will be corrected later)
-        if (not naarr.iscontiguous()):
-            # Do a copy of the array in case is not contiguous
-            naarr = numarray.NDArray.copy(naarr)
-
-        return naarr, flavor
-
-    def _open(self):
+    def _g_open(self):
         """Get the metadata info for an array in file."""
 
-        (self.type, self.stype, self.shape, self.itemsize, self.byteorder,
-         self._v_chunksize) = self._openArray()
-        # Compute the maxTuples
-        self._v_maxTuples = self._v_chunksize * CHUNKTIMES
+        (self._v_objectID, self.type, self.stype, self.shape,
+         self.itemsize, self.byteorder, self._v_chunksize) = self._openArray()
 
         # Get enumeration from disk.
         if self.stype == 'Enum':
             (self._enum, self.type) = self._loadEnum()
-        else:
-            self._enum = None
-
         # Compute the rowsize for each element
         self.rowsize = self.itemsize
         for i in xrange(len(self.shape)):
@@ -322,6 +324,117 @@ Sorry, but this object is not supported.""" % (arr)
         else:
             self.nrows = 1L   # Scalar case
 
+        # Compute the maxTuples for the buffers in iterators
+        (self._v_maxTuples, chunksize) = \
+                            calcBufferSize(self.rowsize, self.nrows)
+
+        return self._v_objectID
+
+    def _convertIntoNA(self, arr):
+        "Convert a generic object into a numarray object"
+
+        if isinstance(arr, numarray.NumArray):
+            flavor = "numarray"
+            if not arr.iscontiguous():
+                # Do a copy of the array in case it is not contiguous
+                #naarr = numarray.array(arr)
+                # We should use the NDArray.copy in order to avoid
+                # changing the byteorder.
+                naarr = numarray.NDArray.copy(arr)
+            else:
+                naarr = arr
+            self.byteorder  = naarr._byteorder
+        elif isinstance(arr, strings.CharArray):
+            flavor = "numarray"
+            naarr = arr
+            self.byteorder = "non-relevant"
+        elif numpy_imported and type(arr) == numpy.ndarray:
+            flavor = "numpy"
+            if arr.dtype.char[0] == "S":
+                self.byteorder = "non-relevant"
+                if arr.shape == ():
+                    # 0-d array will be treated as pure python strings
+                    # because numarray does not support dim-0 chararrays
+                    flavor = "python"
+                    naarr = strings.array(arr.item(), padc='\x00')
+                    # Assign the shape here to save it as a dim-0 object
+                    self.shape = ()
+                else:
+                    naarr = strings.array(arr, itemsize=arr.itemsize,
+                                          padc='\x00')
+            else:
+                if not arr.flags['CONTIGUOUS']:
+                    # Do a copy of the array in case it is not contiguous
+                    naarr = numarray.array(arr)
+                else:
+                    naarr = numarray.asarray(arr)
+                self.byteorder = byteorderDict[arr.dtype.str[0]]
+        elif Numeric_imported and type(arr) == Numeric.ArrayType:
+            flavor = "numeric"
+            self.byteorder  = sys.byteorder  # The only possible for Numeric
+            if arr.typecode() == "c":
+                # To emulate as close as possible Numeric character arrays,
+                # itemsize for chararrays will be always 1
+                if arr.iscontiguous():
+                    # This the fastest way to convert from Numeric to numarray
+                    # because no data copy is involved
+                    naarr = strings.array(buffer(arr),
+                                          itemsize=1,
+                                          shape=arr.shape,
+                                          padc='\x00')
+                else:
+                    # Here we absolutely need a copy so as to obtain a buffer.
+                    # Perhaps this can be avoided or optimized by using
+                    # the tolist() method, but this should be tested.
+                    naarr = strings.array(buffer(arr.copy()),
+                                          itemsize=1,
+                                          shape=arr.shape,
+                                          padc='\x00')
+            else:
+                if arr.iscontiguous():
+                    # This the fastest way to convert from Numeric to numarray
+                    # because no data copy is involved
+                    naarr = numarray.asarray(arr)
+                else:
+                    # Here we absolutely need a copy in order
+                    # to obtain a buffer.
+                    naarr = numarray.array(arr)
+
+        elif (isinstance(arr, tuple) or
+              isinstance(arr, list) or
+              isinstance(arr, int) or
+              isinstance(arr, float) or
+              isinstance(arr, complex)):
+            flavor = "python"
+            # Test if can convert into a numarray object
+            try:
+                naarr = numarray.array(arr)
+            # If not, test with a chararray
+            except TypeError:
+                try:
+                    naarr = strings.array(arr, padc='\x00')
+                # If still doesn't, issues an error
+                except:  #XXX
+                    raise TypeError, \
+"""The object '%s' can't be converted into a numerical or character array.
+Sorry, but this object is not supported.""" % (arr)
+        elif isinstance(arr, str):
+            flavor = "python"
+            naarr = strings.array(arr, padc='\x00')
+            # Assign the shape here to save it as a dim-0 object
+            self.shape = ()
+        else:
+            raise TypeError, \
+"""The object '%s' is not in the list of supported objects: numarray,
+numpy, numeric, homogeneous list or tuple, int, float, complex or str.
+Sorry, but this object is not supported.""" % (arr)
+
+        # We always want a contiguous buffer (no matter if it has an
+        # offset or not; that will be corrected later on)
+
+        return naarr, flavor
+
+
     def getEnum(self):
         """
         Get the enumerated type associated with this array.
@@ -355,7 +468,7 @@ Sorry, but this object is not supported.""" % (arr)
     def __iter__(self):
         """Iterate over all the rows."""
 
-        if not hasattr(self, "_init"):
+        if not self._init:
             # If the iterator is called directly, assign default variables
             self._start = 0
             self._stop = self.nrows
@@ -370,13 +483,13 @@ Sorry, but this object is not supported.""" % (arr)
         self._nrowsread = self._start
         self._startb = self._start
         self._row = -1   # Sentinel
-        self._init = 1    # Sentinel
+        self._init = True  # Sentinel
         self.nrow = self._start - self._step    # row number
 
     def next(self):
         "next() method for __iter__() that is called on each iteration"
         if self._nrowsread >= self._stop:
-            del self._init
+            self._init = False
             raise StopIteration        # end of iteration
         else:
             # Read a chunk of rows
@@ -388,20 +501,31 @@ Sorry, but this object is not supported.""" % (arr)
                 self.listarr = self.read(self._startb, self._stopb, self._step)
                 # Swap the axes to easy the return of elements
                 if self.extdim > 0:
-                    if self.flavor == "Numeric":
+                    if self.flavor == "numpy":
+                        if numpy_imported:
+                            self.listarr = numpy.swapaxes(self.listarr,
+                                                          self.extdim, 0)
+                        else:
+                            # Warn the user
+                            warnings.warn( \
+"""The object on-disk has numpy flavor, but numpy is not installed locally. Returning a numarray object instead!.""")
+                            # Default to numarray
+                            self.listarr = numarray.swapaxes(self.listarr,
+                                                             self.extdim, 0)
+                    elif self.flavor == "numeric":
                         if Numeric_imported:
                             self.listarr = Numeric.swapaxes(self.listarr,
                                                             self.extdim, 0)
                         else:
                             # Warn the user
                             warnings.warn( \
-"""The object on-disk has Numeric flavor, but Numeric is not installed locally. Returning a numarray object instead!.""")
+"""The object on-disk has numeric flavor, but Numeric is not installed locally. Returning a numarray object instead!.""")
                             # Default to numarray
-                            self.listarr = swapaxes(self.listarr,
-                                                    self.extdim, 0)
+                            self.listarr = numarray.swapaxes(self.listarr,
+                                                             self.extdim, 0)
                     else:
                         self.listarr = numarray.swapaxes(self.listarr,
-			                                 self.extdim, 0)
+                                                         self.extdim, 0)
                 self._row = -1
                 self._startb = self._stopb
             self._row += 1
@@ -441,8 +565,8 @@ Sorry, but this object is not supported.""" % (arr)
             elif dim >= maxlen:
                 raise IndexError, "Too many indices for object '%s'" % \
                       self._v_pathname
-            elif type(key) in (int, long):
-                # Index out of range protection
+            elif is_idx(key):
+                # Protection for index out of range
                 if key >= self.shape[dim]:
                     raise IndexError, "Index out of range"
                 if key < 0:
@@ -476,6 +600,7 @@ Sorry, but this object is not supported.""" % (arr)
         for dim in xrange(len(self.shape)):
             new_dim = ((stopl[dim] - startl[dim] - 1) / stepl[dim]) + 1
             if not (new_dim == 1 and stop_None[dim]):
+            #if not stop_None[dim]:
                 # Append dimension
                 shape.append(new_dim)
 
@@ -493,7 +618,6 @@ Sorry, but this object is not supported.""" % (arr)
         """
 
         startl, stopl, stepl, shape = self._interpret_indexing(keys)
-
         return self._readSlice(startl, stopl, stepl, shape)
 
 
@@ -520,48 +644,41 @@ Sorry, but this object is not supported.""" % (arr)
         startl, stopl, stepl, shape = self._interpret_indexing(keys)
         countl = ((stopl - startl - 1) / stepl) + 1
         # Create an array compliant with the specified slice
-        if str(self.type) == "CharType":
+        if self.stype == "CharType":
             narr = strings.array(None, itemsize=self.itemsize,
             # Here shape=shape should be enough, but it makes some
             # tests to fail. This should be analyzed more carefully.
             # F. Altet 2005-09-12
-                                 shape=countl)
+                                 shape=countl, padc='\x00')
+                                 #shape=shape, padc='\x00')
         else:
             narr = numarray.array(None, shape=shape, type=self.type)
 
-        shape = numarray.array(shape, shape=len(shape), type=numarray.Int64)
-
         # Assign the value to it
         try:
             narr[...] = value
-        except:  #XXX
-            (typerr, value2, traceback) = sys.exc_info()
+        except Exception, exc:  #XXX
             raise ValueError, \
 """value parameter '%s' cannot be converted into an array object compliant with %s:
 '%r'
-The error was: <%s>""" % (value, self.__class__.__name__, self, value2)
+The error was: <%s>""" % (value, self.__class__.__name__, self, exc)
 
         if narr.size():
             self._modify(startl, stepl, countl, narr)
 
     # Accessor for the _readArray method in superclass
     def _readSlice(self, startl, stopl, stepl, shape):
-
-        if self.extdim < 0:
-            extdim = 0
-        else:
-            extdim = self.extdim
-
         if repr(self.type) == "CharType":
-            # Workaround for numarray bug #997997
             if shape <> []:
-                arr = strings.array(None, itemsize=self.itemsize, shape=shape)
+                arr = strings.array(None, itemsize=self.itemsize,
+                                    shape=shape, padc='\x00')
             else:
-                arr = strings.array([""], itemsize=self.itemsize, shape=shape)
+                # Avoids returning an array([' '])
+                arr = strings.array([""], itemsize=self.itemsize,
+                                    shape=shape, padc='\x00')
         else:
-            #arr = numarray.zeros(type=self.type, shape=shape)
-            # This is slightly faster (~3%) than zeros()
-            arr = numarray.array(None,type=self.type, shape=shape)
+            arr = self._cache = numarray.array(None,type=self.type,
+                                               shape=shape)
             # Set the same byteorder than on-disk
             arr._byteorder = self.byteorder
 
@@ -570,10 +687,11 @@ The error was: <%s>""" % (value, self.__class__.__name__, self, value2)
             # Arrays that have non-zero dimensionality
             self._g_readSlice(startl, stopl, stepl, arr)
 
-        if hasattr(self, "_v_convert") and self._v_convert == 0:
+        if not self._v_convert:
             return arr
 
-        if self.flavor in ["NumArray", "CharArray"]:
+        # flavors 'NumArray' and 'CharArray' for backward compatibility
+        if self.flavor in ['numarray', 'NumArray', 'CharArray']:
             if arr.shape == ():  # Scalar case
                 return arr[()]
             else:             # No conversion needed
@@ -581,10 +699,6 @@ The error was: <%s>""" % (value, self.__class__.__name__, self, value2)
         # Fixes #968131
         elif arr.shape == ():  # Scalar case
             return arr[()]  # return the value.
-        # The next solution isn't appropriate as a scalar array is
-        # meant as a way to return a Python value
-#         elif arr.shape == (1,):  # Scalar case
-#             return arr[0]  # return the value.
         else:
             return convToFlavor(self, arr)
 
@@ -604,13 +718,8 @@ The error was: <%s>""" % (value, self.__class__.__name__, self, value2)
             shape[extdim] = rowstoread
             shape = tuple(shape)
         if repr(self.type) == "CharType":
-            #arr = strings.array(None, itemsize=self.itemsize, shape=shape)
-            # a workaround for a bug intoduced in numarray 1.3.1
-            if self.shape == ():
-                arr = strings.array(' '*self.itemsize, 
-		                    itemsize=self.itemsize, shape=shape)
-            else:
-                arr = strings.array(None, itemsize=self.itemsize, shape=shape)
+            arr = strings.array(None, itemsize=self.itemsize,
+                                shape=shape, padc='\x00')
         else:
             arr = numarray.array(None, type=self.type, shape=shape)
             # Set the same byteorder than on-disk
@@ -621,7 +730,8 @@ The error was: <%s>""" % (value, self.__class__.__name__, self, value2)
             # Arrays that have non-zero dimensionality
             self._readArray(start, stop, step, arr)
 
-        if self.flavor in ["NumArray", "CharArray"]:
+        # Flavors "NumArray" and "CharArray" remains here for backward compatibility
+        if self.flavor in ["numarray", "NumArray", "CharArray"]:
             # No conversion needed
             return arr
         # Fixes #968131
@@ -630,17 +740,17 @@ The error was: <%s>""" % (value, self.__class__.__name__, self, value2)
         else:
             return convToFlavor(self, arr)
 
-    def _g_copyWithStats(self, group, name, start, stop, step, title, filters):
+    def _g_copyWithStats(self, group, name, start, stop, step,
+                         title, filters, log):
         "Private part of Leaf.copy() for each kind of leaf"
         # Get the slice of the array
         # (non-buffered version)
-	if self.shape:
+        if self.shape:
             arr = self[start:stop:step]
-	else:
-	    arr = self[()]
+        else:
+            arr = self[()]
         # Build the new Array object
-        object = self._v_file.createArray(
-            group, name, arr, title=title, _log = False)
+        object = Array(group, name, arr, title=title, log=log)
         nbytes = self.itemsize
         for i in self.shape:
             nbytes*=i
@@ -661,7 +771,6 @@ The error was: <%s>""" % (value, self.__class__.__name__, self, value2)
                        self.nrows, self.flavor, self.byteorder)
 
 
-
 class ImageArray(Array):
 
     """
