@@ -4,8 +4,7 @@
 #       Created: May 26, 2003
 #       Author:  Francesc Altet - faltet@carabos.com
 #
-#       $Source: /home/ivan/_/programari/pytables/svn/cvs/pytables/pytables/tables/AttributeSet.py,v $
-#       $Id: AttributeSet.py 1190 2005-09-13 12:02:32Z faltet $
+#       $Id: AttributeSet.py 1481 2006-02-23 13:01:51Z faltet $
 #
 ########################################################################
 
@@ -37,13 +36,13 @@ import numarray
 import tables.hdf5Extension as hdf5Extension
 from tables.constants import MAX_NODE_ATTRS
 from tables.registry import classNameDict
-from tables.exceptions import PerformanceWarning
+from tables.exceptions import ClosedNodeError, PerformanceWarning
 from tables.utils import checkNameValidity
 from tables.undoredo import attrToShadow
 
 
 
-__version__ = "$Revision: 1190 $"
+__version__ = "$Revision: 1481 $"
 
 # System attributes
 SYS_ATTRS = ["CLASS", "VERSION", "TITLE", "NROWS", "EXTDIM",
@@ -117,6 +116,12 @@ class AttributeSet(hdf5Extension.AttributeSet, object):
 
     """
 
+    def _g_getnode(self):
+        return self._v__nodeFile._getNode(self._v__nodePath)
+
+    _v_node = property(_g_getnode)
+
+
     def __init__(self, node):
         """Create the basic structures to keep the attribute information.
 
@@ -126,11 +131,20 @@ class AttributeSet(hdf5Extension.AttributeSet, object):
 
         """
 
+        node._g_checkOpen()
+
         mydict = self.__dict__
 
         self._g_new(node)
-        mydict["_v_node"] = node
+        mydict["_v__nodeFile"] = node._v_file
+        mydict["_v__nodePath"] = node._v_pathname
         mydict["_v_attrnames"] = self._g_listAttr()
+        # Get the file version format. This is an optimization
+        # in order to avoid accessing too much to it.
+        if hasattr(node._v_file, "format_version"):
+            mydict["_v__format_version"] = node._v_file.format_version
+        else:
+            mydict["_v__format_version"] = None
         # Split the attribute list in system and user lists
         mydict["_v_attrnamessys"] = []
         mydict["_v_attrnamesuser"] = []
@@ -148,6 +162,31 @@ class AttributeSet(hdf5Extension.AttributeSet, object):
         self._v_attrnamessys.sort()
         self._v_attrnamesuser.sort()
 
+
+    def _g_updateNodeLocation(self, node):
+        """Updates the location information about the associated `node`."""
+
+        myDict = self.__dict__
+        myDict['_v__nodeFile'] = node._v_file
+        myDict['_v__nodePath'] = node._v_pathname
+        # hdf5Extension operations:
+        self._g_new(node)
+
+
+    def _g_checkOpen(self):
+        """
+        Check that the attribute set is open.
+
+        If the attribute set is closed, a `ClosedNodeError` is raised.
+        """
+
+        if '_v__nodePath' not in self.__dict__:
+            raise ClosedNodeError("the attribute set is closed")
+        assert self._v_node._f_isOpen(), \
+               "found an open attribute set of a closed node"
+
+
+
     def _f_list(self, attrset="user"):
         """_f_list(attrset = 'user') -> list.  List of attribute names.
 
@@ -159,6 +198,8 @@ class AttributeSet(hdf5Extension.AttributeSet, object):
         Finally, 'all' returns both the system and user attributes.
         """
 
+        self._g_checkOpen()
+
         if attrset == "user":
             return self._v_attrnamesuser[:]
         elif attrset == "sys":
@@ -166,38 +207,32 @@ class AttributeSet(hdf5Extension.AttributeSet, object):
         elif attrset == "all":
             return self._v_attrnames[:]
 
+
     def __getattr__(self, name):
         """Get the attribute named "name"."""
 
+        self._g_checkOpen()
+
         # If attribute does not exist, raise AttributeError
         if not name in self._v_attrnames:
             raise AttributeError, \
                   "Attribute '%s' does not exist in node: '%s'" % \
-                  (name, self._v_node._v_pathname)
+                  (name, self._v__nodePath)
 
-        # Get the file version format
-        #print "node, attr-->", self._v_node._v_pathname, name
-#         if self._v_node._v_pathname == "/":
-#             raise Exception
-        if hasattr( self._v_node._v_file, "format_version"):
-            format_version = self._v_node._v_file.format_version
-        else:
-            format_version = None
         # Read the attribute from disk. This is an optimization to read
         # quickly system attributes that are _string_ values, but it
         # takes care of other types as well as for example NROWS for
         # Tables and EXTDIM for EArrays
+        format_version = self._v__format_version
         if format_version is not None and format_version < "1.4":
             value = self._g_getAttr(name)   # Takes 1.3s/3.7s
         else:
-            if issysattrname(name):
+            if issysattrname(name) and not name.endswith("FILL"):
                 if name in ["NROWS", "EXTDIM", "AUTOMATIC_INDEX",
                             "REINDEX", "DIRTY", "NODE_TYPE_VERSION"]:
                     # These are Int32 or Int64 integers
                     value = self._g_getAttr(name)[()]
                 else:
-                    # _g_getSysAttr works only for string attributes
-                    # with length less than 256 bytes
                     value = self._g_getSysAttr(name)   # Takes only 0.6s/2.9s
             else:
                 # This the general way to read attributes (takes more time)
@@ -291,6 +326,8 @@ class AttributeSet(hdf5Extension.AttributeSet, object):
         number of attributes in a node is going to be exceeded.
         """
 
+        self._g_checkOpen()
+
         node = self._v_node
         nodeFile = node._v_file
         nodePathname = node._v_pathname
@@ -299,6 +336,8 @@ class AttributeSet(hdf5Extension.AttributeSet, object):
         # Check for name validity
         checkNameValidity(name)
 
+        nodeFile._checkWritable()
+
         # Check that the attribute is not a system one (read-only)
         ##if name in RO_ATTRS:
         ##    raise AttributeError, \
@@ -361,6 +400,8 @@ be ready to see PyTables asking for *lots* of memory and possibly slow I/O"""
         specified, an ``AttributeError`` is raised.
         """
 
+        self._g_checkOpen()
+
         node = self._v_node
         nodeFile = node._v_file
 
@@ -375,6 +416,8 @@ be ready to see PyTables asking for *lots* of memory and possibly slow I/O"""
             raise AttributeError, \
                   "System attribute ('%s') cannot be deleted" % (name)
 
+        nodeFile._checkWritable()
+
         # Remove the PyTables attribute or move it to shadow.
         if nodeFile.isUndoEnabled():
             nodePathname = node._v_pathname
@@ -392,12 +435,15 @@ be ready to see PyTables asking for *lots* of memory and possibly slow I/O"""
         Returns ``True`` if the attribute set has an attribute with the
         given name, ``False`` otherwise.
         """
+        self._g_checkOpen()
         return name in self._v_attrnames
 
 
     def _f_rename(self, oldattrname, newattrname):
         "Rename an attribute"
 
+        self._g_checkOpen()
+
         if oldattrname == newattrname:
             # Do nothing
             return
@@ -452,6 +498,8 @@ be ready to see PyTables asking for *lots* of memory and possibly slow I/O"""
         given PyTables node, replacing the existing ones.
         """
 
+        self._g_checkOpen()
+
         # AttributeSet must be defined in order to define a Node.
         # However, we need to know Node here.
         # Using classNameDict avoids a circular import.
@@ -461,32 +509,41 @@ be ready to see PyTables asking for *lots* of memory and possibly slow I/O"""
 
 
     def _f_close(self):
-        "Delete some back-references"
-        del self.__dict__["_v_node"]
-        # After the objects are disconnected, destroy the
-        # object dictionary using the brute force ;-)
-        # This should help to the garbage collector
-        #self.__dict__.clear()
+        self.__dict__.clear()
 
-        pass
 
     def __str__(self):
         """The string representation for this object."""
+
+        node = self._v_node
+        if node is None or not node._f_isOpen():
+            return repr(self)
+
         # Get the associated filename
-        filename = self._v_node._v_file.filename
+        filename = node._v_file.filename
         # The pathname
-        pathname = self._v_node._v_pathname
+        pathname = node._v_pathname
         # Get this class name
         classname = self.__class__.__name__
-        # The attrribute names
-        attrnumber = len([ n for n in self._v_attrnames if not issysattrname(n) ])
+        # The attribute names
+        #attrnumber = len([ n for n in self._v_attrnames if not issysattrname(n) ])
+        # Showing all attributes by default
+        attrnumber = len([ n for n in self._v_attrnames ])
         return "%s._v_attrs (%s), %s attributes" % (pathname, classname, attrnumber)
 
     def __repr__(self):
         """A detailed string representation for this object."""
 
+        node = self._v_node
+        if node is None:
+            return "<closed AttributeSet>"
+        if not node._f_isOpen():
+            return "<AttributeSet of closed Node>"
+
         # print additional info only if there are attributes to show
-        attrnames = [ n for n in self._v_attrnames if not issysattrname(n) ]
+        #attrnames = [ n for n in self._v_attrnames if not issysattrname(n) ]
+        # Showing all attributes by default
+        attrnames = [ n for n in self._v_attrnames ]
         if len(attrnames):
             rep = [ '%s := %r' %  (attr, getattr(self, attr) )
                     for attr in attrnames ]
