@@ -4,8 +4,7 @@
 #       Created: June 08, 2004
 #       Author:  Francesc Altet - faltet@carabos.com
 #
-#       $Source: /home/ivan/_/programari/pytables/svn/cvs/pytables/pytables/tables/Index.py,v $
-#       $Id: Index.py 1015 2005-06-17 17:55:14Z faltet $
+#       $Id: Index.py 1672 2006-06-19 12:24:20Z faltet $
 #
 ########################################################################
 
@@ -15,6 +14,7 @@ See Index class docstring for more info.
 
 Classes:
 
+    IndexProps
     Index
 
 Functions:
@@ -42,7 +42,7 @@ from tables.IndexArray import IndexArray
 from tables.Group import Group
 from tables.utils import joinPath
 
-__version__ = "$Revision: 1015 $"
+__version__ = "$Revision: 1672 $"
 
 # default version for INDEX objects
 #obversion = "1.0"    # initial version
@@ -312,7 +312,6 @@ class IndexProps(object):
         return repr(self)
 
 class Index(hdf5Extension.Index, Group):
-    _c_classId = 'CINDEX'
 
     """Represent the index (sorted and reverse index) dataset in HDF5 file.
 
@@ -330,23 +329,54 @@ class Index(hdf5Extension.Index, Group):
     Instance variables:
 
         column -- The column object this index belongs to
-        type -- The type class for the index.
-        itemsize -- The size of the atomic items. Specially useful for
-            CharArrays.
-        nrows -- The number of slices in index.
-        nelements -- The total number of elements in the index.
-        nelemslice -- The number of elements per slice.
-        chunksize -- The HDF5 chunksize for each slice.
-        filters -- The Filters instance for this object.
         dirty -- Whether the index is dirty or not.
+        nrows -- The number of slices in the index.
+        nelemslice -- The number of elements per slice.
+        nelements -- The number of indexed rows.
+        shape -- The shape of this index (in slices and elements).
+        filters -- The properties used to filter the stored items.
         sorted -- The IndexArray object with the sorted values information.
         indices -- The IndexArray object with the sorted indices information.
 
     """
 
-    def __init__(self, atom = None, column = None, name = None,
-                 title = "", filters = None, expectedrows = 1000,
-                 testmode = 0, new = True):
+    _c_classId = 'CINDEX'
+
+
+    # <properties>
+
+    dirty = property(
+        lambda self: self.column.dirty, None, None,
+        "Whether the index is dirty or not.")
+
+    nrows = property(
+        lambda self: self.sorted.nrows, None, None,
+        "The number of slices in the index.")
+
+    nelemslice = property(
+        lambda self: self.sorted.nelemslice, None, None,
+        "The number of elements per slice.")
+
+    nelements = property(
+        lambda self: self.sorted.nrows * self.sorted.nelemslice, None, None,
+        "The number of indexed rows.")
+
+    shape = property(
+        lambda self: (self.sorted.nrows, self.sorted.nelemslice), None, None,
+        "The shape of this index (in slices and elements).")
+
+    filters = property(
+        lambda self: self.sorted.filters, None, None,
+        "The properties used to filter the stored items.")
+
+    # </properties>
+
+
+    def __init__(self, parentNode, name,
+                 atom=None, column=None,
+                 title="", filters=None,
+                 expectedrows=0,
+                 testmode=False, new=True):
         """Create an Index instance.
 
         Keyword arguments:
@@ -357,8 +387,6 @@ class Index(hdf5Extension.Index, Group):
 
         column -- The column object to be indexed
 
-        name -- The name for this Index object.
-
         title -- Sets a TITLE attribute of the Index entity.
 
         filters -- An instance of the Filters class that provides
@@ -369,125 +397,90 @@ class Index(hdf5Extension.Index, Group):
 
         expectedrows -- Represents an user estimate about the number
             of row slices that will be added to the growable dimension
-            in the IndexArray object. If not provided, the default
-            value is 1000 slices.
+            in the IndexArray object.
 
         """
-        super(Index, self).__init__(title, new, filters)
 
-        mydict = self.__dict__
+        self._v_version = None
+        """The object version of this index."""
 
-        # The next are needed by the Group protocol
-        mydict['name'] = name
-        mydict['_v_expectedrows'] = expectedrows
-        mydict['_v_version'] = obversion
+        self._v_expectedrows = expectedrows
+        """The expected number of items of index arrays."""
+        self.testmode = testmode
+        """Enables test mode for index chunk size calculation."""
+        self.atom = atom
+        """The `Atom` instance matching to be stored by the index array."""
+        self.column = column
+        """The `Column` instance for the indexed column."""
 
-        # Collect info in case the index is to be created
-        if new:
-            mydict['atom'] = atom
-            mydict['column'] = column
+        self.starts = None
+        """Where the values fulfiling conditions starts for every slice."""
+        self.lengths = None
+        """Lengths of the values fulfilling conditions for every slice."""
 
-        mydict['testmode'] = testmode
+        # Index creation is never logged.
+        super(Index, self).__init__(
+            parentNode, name, title, new, filters, log=False)
 
-    def _g_create(self):
-        """Call the createGroup method in extension to create the
-        group on disk. Also set attributes for this group. Finally,
-        add the index information as IndexArrays."""
+        # Set the version number of this object as an index, not a group.
+        self._v_version = obversion
 
-        global obversion
 
-        mydict = self.__dict__
-        # Create the Index Group
-        mydict['_v_objectID'] =  self._g_createGroup()
+    def _g_postInitHook(self):
+        super(Index, self)._g_postInitHook()
+
+        # Index arrays must only be created for new indexes.
+        if not self._v_new:
+            return
 
         # Set the filters for this object (they are *not* inherited)
-        if self._v_new_filters is None:
+        filters = self._v_new_filters
+        if filters is None:
             # If not filters has been passed in the constructor,
             # set a sensible default, using zlib compression and shuffling
             filters = Filters(complevel = 1, complib = "zlib",
                               shuffle = 1, fletcher32 = 0)
-        else:
-            filters = self._v_new_filters
 
         # Create the IndexArray for sorted values
-        sorted = IndexArray(self, self.atom,
-                            "Sorted Values", filters,
-                            self._v_expectedrows, self.testmode)
-        # Register the sorted values array
-        setattr(self, "sorted", sorted)
+        IndexArray(self, 'sorted',
+                   self.atom, "Sorted Values", filters,
+                   self.testmode, self._v_expectedrows)
 
         # Create the IndexArray for index values
-        indices = IndexArray(self, Atom("Int32", shape=1),
-                             "Reverse Indices", filters,
-                             self._v_expectedrows, self.testmode)
-        # Register the indices array
-        setattr(self, "indices", indices)
+        IndexArray(self, 'indices',
+                   Atom("Int32", shape=(0, 1)), "Reverse Indices", filters,
+                   self.testmode, self._v_expectedrows)
 
-        # get the remaining variables
-        self._getCompleteObject()
 
-    def _g_afterOpen(self):
-        """Tidy object tree after its creation."""
+    def _g_updateDependent(self):
+        super(Index, self)._g_updateDependent()
+        self.column._updateIndexLocation(self)
 
-        # Call the post-open methods in sorted and indices
-        # This has to executed *before* _getCompleteObject
-        self.sorted._g_afterOpen()
-        self.indices._g_afterOpen()
-
-        # get the remaining variables
-        self._getCompleteObject()
-
-    def _getCompleteObject(self):
-        "Get the remaining variables for a complete Index object"
-
-        mydict = self.__dict__
-        mydict['type'] = self.sorted.type
-        mydict['itemsize'] = self.sorted.itemsize
-        mydict['chunksize'] = self.sorted.chunksize
-        mydict['byteorder'] = self.sorted.byteorder
-        mydict['nrows'] = self.sorted.nrows
-        mydict['nelemslice'] = self.sorted.nelemslice
-        mydict['nelements'] = self.nrows * self.nelemslice
-        mydict['shape'] = (self.nrows, self.nelemslice)
-        mydict['filters'] = self.sorted.filters
 
     def append(self, arr):
         """Append the object to this (enlargeable) object"""
 
+        # Objects that arrive here should be numarray objects already
         # Save the sorted array
         if str(self.sorted.type) == "CharType":
             s=arr.argsort()
-            # Caveat: this conversion is necessary for portability on
-            # 64-bit systems because indexes are 64-bit long on these
-            # platforms
-            self.indices.append(numarray.array(s, type="Int32"))
-            self.sorted.append(arr[s])
         else:
-            #self.sorted.append(numarray.sort(arr))
-            #self.indices.append(numarray.argsort(arr))
-            # The next is a 10% faster, but the ideal solution would
-            # be to find a funtion in numarray that returns both
-            # sorted and argsorted all in one call
             s=numarray.argsort(arr)
-            # Caveat: this conversion is necessary for portability on
-            # 64-bit systems because indexes are 64-bit long on these
-            # platforms
-            self.indices.append(numarray.array(s, type="Int32"))
-            self.sorted.append(arr[s])
-        # Update nrows after a successful append
-        mydict = self.__dict__
-        mydict['nrows'] = self.sorted.nrows
-        mydict['nelements'] = self.nrows * self.nelemslice
-        mydict['shape'] = (self.nrows, self.nelemslice)
+        # Caveat: this conversion is necessary for portability on
+        # 64-bit systems because indexes are 64-bit long on these
+        # platforms
+        self.indices.append(numarray.array(s, type="Int32"))
+        #self.sorted.append(numarray.asarray(arr[s])) # arr can be numpy
+        self.sorted.append(arr[s]) # arr can be numpy
+
 
     def search(self, item):
         """Do a binary search in this index for an item"""
         #t1=time.time()
         ntotaliter = 0; tlen = 0
-        mydict = self.__dict__
-        mydict['starts'] = []; mydict['lengths'] = []
+        self.starts = []; self.lengths = []
         #self.irow = 0; self.len1 = 0; self.len2 = 0;  # useful for getCoords()
-        self.sorted._initSortedSlice(self.chunksize)
+        self.sorted._initSortedSlice(self.sorted.chunksize)
         # Do the lookup for values fullfilling the conditions
         for i in xrange(self.sorted.nrows):
             (start, stop, niter) = self.sorted._searchBin(i, item)
@@ -513,7 +506,7 @@ class Index(hdf5Extension.Index, Group):
         #t1=time.time()
         len1 = 0; len2 = 0; relCoords = 0
         # Correction against asking too many elements
-        nindexedrows = self.nelemslice*self.nrows
+        nindexedrows = self.nelements
         if startCoords + maxCoords > nindexedrows:
             maxCoords = nindexedrows - startCoords
         for irow in xrange(self.sorted.nrows):
@@ -549,7 +542,7 @@ class Index(hdf5Extension.Index, Group):
 #         """Get the coordinates of indices satisfiying the cuts"""
 #         relCoords = 0
 #         # Correction against asking too many elements
-#         nindexedrows = self.nelemslice*self.nrows
+#         nindexedrows = self.nelements
 #         if startCoords + maxCoords > nindexedrows:
 #             maxCoords = nindexedrows - startCoords
 #         #for irow in xrange(self.irow, self.sorted.nrows):
@@ -677,7 +670,7 @@ val1 must be less or equal than val2""")
         #print "time reading indices:", time.time()-t1
         return ncoords
 
-    def _remove(self):
+    def _f_remove(self, recursive=False):
         """Remove this Index object"""
 
         if utilsExtension.whichLibVersion("hdf5")[1] == "1.6.3":
@@ -688,46 +681,20 @@ dataset. If you are getting such a seg fault immediately after this
 message, please, get a patched version of HDF5 1.6.3, or, better,
 get HDF5 1.6.4.""")
 
-        # Delete the associated IndexArrays
-        self.sorted.remove()
-        self.indices.remove()
-        # delete some references
-        mydict = self.__dict__
-        mydict['indices'] = None
-        mydict['sorted'] = None
-        # delete self
-        self._f_remove()
+        # Index removal is always recursive,
+        # no matter what `recursive` says.
+        super(Index, self)._f_remove(True)
 
-    def _f_close(self):
-        # Check that the index is not already closed
-        if self._v_file == None:
-            return
-        # close the indices (might be already removed by self._g_remove!)
-        if hasattr(self,"sorted") and self.sorted <> None:
-            self.sorted.close()
-        if hasattr(self,"indices") and self.indices <> None:
-            self.indices.close()
-        # delete some references
-        mydict = self.__dict__
-        mydict['atom'] = None
-        mydict['column'] = None
-        mydict['filters'] = None
-        mydict['indices'] = None
-        mydict['sorted'] = None
-        #mydict['_v_attrs'] = None  # This is deleted in Group._f_close()
-        # Call the superclass close method
-        Group._f_close(self)
 
     def __str__(self):
         """This provides a more compact representation than __repr__"""
         return "Index(%s, shape=%s, chunksize=%s)" % \
-               (self.nelements, self.shape, self.chunksize)
+               (self.nelements, self.shape, self.sorted.chunksize)
 
     def __repr__(self):
         """This provides more metainfo than standard __repr__"""
 
         cpathname = self.column.table._v_pathname + ".cols." + self.column.name
-        dirty = self.column.dirty
         return """%s (Index for column %s)
   type := %r
   nelements := %s
@@ -738,6 +705,6 @@ get HDF5 1.6.4.""")
   dirty := %s
   sorted := %s
   indices := %s""" % (self._v_pathname, cpathname,
-                     self.type, self.nelements, self.shape,
+                     self.sorted.type, self.nelements, self.shape,
                      self.sorted.chunksize, self.sorted.byteorder,
-                     self.filters, dirty, self.sorted, self.indices)
+                     self.filters, self.dirty, self.sorted, self.indices)
