@@ -4,8 +4,7 @@
 #       Created: June 02, 2004
 #       Author:  Francesc Altet - faltet@carabos.com
 #
-#       $Source: /home/ivan/_/programari/pytables/svn/cvs/pytables/pytables/tables/IndexArray.py,v $
-#       $Id: IndexArray.py 1009 2005-06-15 13:39:15Z faltet $
+#       $Id: IndexArray.py 1496 2006-03-13 09:48:13Z faltet $
 #
 ########################################################################
 
@@ -39,7 +38,7 @@ from tables.Atom import Atom, StringAtom
 from tables.EArray import EArray
 
 
-__version__ = "$Revision: 1009 $"
+__version__ = "$Revision: 1496 $"
 
 # default version for IndexARRAY objects
 obversion = "1.0"    # initial version
@@ -48,7 +47,7 @@ obversion = "1.0"    # initial version
 minRowIndex = 10
 
 
-def calcChunksize(expectedrows, testmode=0):
+def calcChunksize(expectedrows, testmode=False):
     """Calculate the HDF5 chunk size for index and sorted arrays.
 
     The logic to do that is based purely in experiments playing with
@@ -85,17 +84,17 @@ def calcChunksize(expectedrows, testmode=0):
     if expKrows < 0.1: # expected rows < 100 thousand
         nelemslice = 5000  # (best experimental)
         chunksize = 1000
-#         nelemslice = 5*1024 
+#         nelemslice = 5*1024
 #         chunksize = 1024
     elif expKrows < 1: # expected rows < 1 milion
         nelemslice = 20000  # (best experimental)
         chunksize = 2000   # (best experimental)
-#         nelemslice = 5*1024 
+#         nelemslice = 5*1024
 #         chunksize = 1024
 #         chunksize = 2048  # (best experimental)
 #         nelemslice = 10*chunksize   # (best experimental)
        #chunksize = 2048
-       #nelemslice = 10*chunksize 
+       #nelemslice = 10*chunksize
     elif expKrows < 10:  # expected rows < 10 milion
         #nelemslice = 500000  # > 1/20th (best for best case)
         #chunksize = 5000  # Experimental (best for best case)
@@ -124,8 +123,9 @@ def calcChunksize(expectedrows, testmode=0):
     #print "nelemslice, chunksize:", (nelemslice, chunksize)
     return (nelemslice, chunksize)
 
+
 class IndexArray(hdf5Extension.IndexArray, EArray):
-    _c_classId = "INDEXARRAY"
+
     """Represent the index (sorted or reverse index) dataset in HDF5 file.
 
     All Numeric and numarray typecodes are supported except for complex
@@ -154,19 +154,22 @@ class IndexArray(hdf5Extension.IndexArray, EArray):
         extdim -- The enlargeable dimension (always the first, or 0).
         nrows -- The number of slices in index.
         nelemslice -- The number of elements per slice.
-        chunksize -- The HDF5 chunksize for the slice dimension (the 1).
+        chunksize -- The HDF5 chunksize for the slice dimension (the second).
 
     """
 
-    def __init__(self, parent = None, atom = None, title = "",
-                 filters = None, expectedrows = 1000000,
-                 testmode=0):
+    _c_classId = "INDEXARRAY"
+
+
+    def __init__(self, parentNode, name,
+                 atom=None, title="",
+                 filters=None,
+                 testmode=False,
+                 expectedrows=0):
         """Create an IndexArray instance.
 
         Keyword arguments:
 
-        parent -- The Index class from which this object will hang off
-
         atom -- An Atom object representing the shape, type and flavor
             of the atomic objects to be saved. Only scalar atoms are
             supported.
@@ -178,74 +181,62 @@ class IndexArray(hdf5Extension.IndexArray, EArray):
             during the life of this object.
 
         expectedrows -- Represents an user estimate about the number
-            of elements to index. If not provided, the default
-            value is 1000000 slices.
+            of elements to index.
 
         """
-        self._v_parent = parent
-        self._v_new_title = title
-        self._v_new_filters = filters
-        self._v_expectedrows = expectedrows
+
         self.testmode = testmode
-        self.flavor = "NumArray"  # Needed by Array methods
-        # Check if we have to create a new object or read their contents
-        # from disk
+        """Enables test mode for index chunk size calculation."""
+        self.nelemslice = None
+        """The number of elements per slice."""
+        self.chunksize = None
+        """The HDF5 chunksize for the slice dimension (the second)."""
+
+        # Compute the optimum number of slices and chunk sizes
+        # for newly created index arrays.
         if atom is not None:
-            self._v_new = 1
-            self.atom = atom
-        else:
-            self._v_new = 0
-            self._after = False
+            (self.nelemslice, self.chunksize) = (
+                calcChunksize(expectedrows, testmode))
 
-    def _create(self):
-        """Save a fresh array (i.e., not present on HDF5 file)."""
+        # Index creation is never logged.
+        super(IndexArray, self).__init__(
+            parentNode, name, atom, title, filters, expectedrows, log=False)
 
-        # All this will eventually end up in the node constructor.
 
-        assert isinstance(self.atom, Atom), "The object passed to the IndexArray constructor must be a descendent of the Atom class."
-        assert self.atom.shape == 1, "Only scalar columns can be indexed."
-        # Version, type, shape, flavor, byteorder
-        self._v_version = obversion
-        self.type = self.atom.type
-        self.stype = self.atom.stype
-        if self.type == "CharType" or isinstance(self.type, records.Char):
-            self.byteorder = "non-relevant"
-        else:
-            # Only support for creating objects in system byteorder
-            self.byteorder  = sys.byteorder
-        # Compute the optimal chunksize
-        (self.nelemslice, self.chunksize) = \
-                          calcChunksize(self._v_expectedrows,
-                                        testmode=self.testmode)
-        # The next is needed by hdf5Extension.Array._createEArray
-        self._v_chunksize = (1, self.chunksize)
-        self.nrows = 0   # No rows initially
-        self.itemsize = self.atom.itemsize
-        self.rowsize = self.atom.atomsize() * self.nelemslice
+    def _g_create(self):
+        assert self.atom.shape == (0, 1), "only scalar columns can be indexed"
+        objectId = super(IndexArray, self)._g_create()
+        assert self.extdim == 0, "computed extendable dimension is wrong"
+        assert self.shape == (0, self.nelemslice), "invalid shape"
+        assert self._v_chunksize == (1, self.chunksize), "invalid chunk size"
+        return objectId
+
+
+    def _calcTuplesAndChunks(self, atom, extdim, expectedrows, compress):
+        return (0, (1, self.chunksize))  # (_v_maxTuples, _v_chunksize)
+
+
+    def _createEArray(self, title):
+        # The shape of the index array needs to be fixed before creating it.
         self.shape = (0, self.nelemslice)
+        super(IndexArray, self)._createEArray(title)
 
-        # extdim computation
-        self.extdim = 0
-        # Create the IndexArray
-        self._createEArray(self._v_new_title)
 
-    def _g_afterOpen(self):
-        """Tidy object tree after its creation."""
-
-        # Protection to avoid executing this twice
-        if self._after == False:
-            # Add some variables that are proper to IndexArray objects
-            self.chunksize = self._v_chunksize[1]  # Get the second dim
+    def _g_postInitHook(self):
+        # Set ``nelemslice`` and ``chunksize`` when opening an existing node;
+        # otherwise, they are already set.
+        if not self._v_new:
             self.nelemslice = self.shape[1]
-            # Compute the rowsize for each element
-            self.rowsize = self.atom.atomsize() * self.nelemslice
-            self._after = True
+            self.chunksize = self._v_chunksize[1]
+        super(IndexArray, self)._g_postInitHook()
+
 
     def append(self, arr):
         """Append the object to this (enlargeable) object"""
         arr.shape = (1, arr.shape[0])
         self._append(arr)
 
+
     # This is coded in pyrex as well, but the improvement in speed is very
     # little. So, it's better to let _searchBin live here.
     def _searchBin(self, nrow, item):
@@ -285,7 +276,7 @@ class IndexArray(hdf5Extension.IndexArray, EArray):
                 result2 = hi - chunksize + result2
         if item1done and item2done:
             return (result1, result2, niter)
-    
+
         # Finally, do a lookup for item1 and item2 if they were not found
         # Lookup in the middle of slice for item1
         if not item1done:
@@ -303,7 +294,7 @@ class IndexArray(hdf5Extension.IndexArray, EArray):
                     break
                 else:
                     hi = result1        # one chunk to the left
-                    lo = hi - chunksize  
+                    lo = hi - chunksize
             result1 = tmpresult1
         # Lookup in the middle of slice for item1
         if not item2done:
@@ -326,23 +317,11 @@ class IndexArray(hdf5Extension.IndexArray, EArray):
             niter = niter + iter
         return (result1, result2, niter)
 
-    def _close(self):
-        """Close this object and exit"""
-        # First, flush the buffers:
-        self.flush()
-        # Delete back references
-        del self._v_parent
-        del self._v_file
-        del self.stype
-        del self.type
-        del self.atom
-        del self.filters
-        self.__dict__.clear()
 
     def __str__(self):
         "A compact representation of this class"
-        return "IndexArray(path=%s)" % \
-               (self._v_parent._g_join(self.name))
+        return "IndexArray(path=%s)" % self._v_pathname
+
 
     def __repr__(self):
         """A verbose representation of this class"""
