@@ -5,7 +5,7 @@
 #       Author:  Francesc Altet - faltet@carabos.com
 #
 #       $Source: /home/ivan/_/programari/pytables/svn/cvs/pytables/pytables/tables/IsDescription.py,v $
-#       $Id: IsDescription.py 1009 2005-06-15 13:39:15Z faltet $
+#       $Id: IsDescription.py 1570 2006-05-02 12:58:22Z faltet $
 #
 ########################################################################
 
@@ -30,13 +30,14 @@ import operator
 
 import numarray as NA
 import numarray.records as records
+import numarray.strings as strings
 
 from tables.enum import Enum
 from tables.utils import checkNameValidity
 import tables
 
 
-__version__ = "$Revision: 1009 $"
+__version__ = "$Revision: 1570 $"
 
 
 # Translation tables for numarray datatypes and recarray formats
@@ -103,6 +104,14 @@ class ShapeMixin:
 
 
 
+def setDefaultString(dflt):
+    "Set a default value for strings. Valid for Col and StringCol instances."
+
+    if dflt != None and not (type(dflt) in [str, list, tuple] or
+                             type(dflt) == strings.CharArray):
+        raise ValueError("Invalid default value: %s" % (dflt,))
+
+
 class Col(ShapeMixin, object):
     "Defines a general column that supports all numarray data types."
 
@@ -138,7 +147,33 @@ class Col(ShapeMixin, object):
 
     def _setDefault(self, dflt):
         "Sets the 'dflt' attribute."
-        self.dflt = dflt
+        # Create NumArray or CharArray objects as defaults
+        # This is better in order to serialize then as attributes
+        if self.type == records.CharType:
+            # We should raise an error in case dflt is None, but that
+            # makes several tests in test_create.py to fail
+            # and I don't have time right now to look into this.
+##            if dflt is None:
+##                raise ValueError, \
+##"You must set a default value for CharType when using the Col constructor."
+##            elif dflt == "":
+##                # Putting just to "" gives problems with numarray
+##                dflt = "\x00"*self.itemsize
+            if dflt is None or dflt == "":
+                # Putting just to "" gives problems with numarray
+                dflt = "\x00"
+            # It is important to set the padding character to NULL in order
+            # to avoid the '' string to become a ' ' after de-serializing.
+            # See:
+            # http://sourceforge.net/tracker/index.php?func=detail&aid=1304615&group_id=1369&atid=450446
+            # for more info.
+            self.dflt = strings.array(dflt, padc='\x00')
+        else:
+            if dflt is None:
+                # We set the value to int(0), but it will be changed
+                # to the appropriate type immediately after.
+                dflt = 0
+            self.dflt = NA.array(dflt, type=self.type)
 
     def _setIndex(self, indexed):
         if indexed and self.type in (NA.Complex32, NA.Complex64):
@@ -150,7 +185,7 @@ class Col(ShapeMixin, object):
 
         # Override itemsize; strings still need some tampering with the shape
         #
-        # This exposes NumPy BUG#1087158, since the comparison is True
+        # This exposes Numeric BUG#1087158, since the comparison is True
         # for self.type = Float64, Int32, Complex64...
         # ivilata(2004-12-17)
         #
@@ -206,6 +241,63 @@ class Col(ShapeMixin, object):
 
 
 
+class StringCol(Col):
+    "Defines a string column."
+
+    def _setType(self, type_):
+        self.type       = records.CharType
+        self.stype      = str(records.CharType)
+
+    def _setShape(self, shape):
+        super(Col, self)._setShape(shape)
+
+        # Set itemsize; forced to None to get it from the 'length' argument
+        self.itemsize = None
+
+    def _setDefault(self, dflt):
+        "Sets the 'dflt' attribute (overwrites the method in Col)."
+        if dflt is None or dflt == "":
+            dflt = "\x00"*self.itemsize
+        # It is important to set the padding character to NULL in order
+        # to avoid the '' string to become a ' ' after de-serializing.
+        # See:
+        # http://sourceforge.net/tracker/index.php?func=detail&aid=1304615&group_id=1369&atid=450446
+        # for more info.
+        self.dflt = strings.array(dflt, padc='\x00')
+
+    def __init__(self, length = None, dflt = None, shape = 1, pos = None,
+                 indexed = False):
+
+        # Some more work needed for constructor call idiosincracies:
+        # 'itemsize' is deduced from the default value if not specified.
+        if length is None and dflt:
+            length = len(dflt)  # 'dflt' has already been checked
+            # We explicitely forbid 0-sized arrays
+            # (this creates too much headaches)
+            if length == 0:
+                length = 1
+
+        if length is None:
+            raise ValueError("""\
+You must specify at least a length or a default value
+  where this length can be inferred from.""")
+
+        # This is set here just to be used in _setDefault()
+        self.itemsize = length
+
+        Col.__init__(
+            self, dtype = 'CharType',
+            dflt = dflt, shape = shape, pos = pos, indexed = indexed)
+        # This needs to be set again (more indiosincracies :-()
+        self.itemsize = length
+        self.recarrtype = revrecarrfmt[self.type]+str(self.itemsize)
+
+    def __repr__(self):
+        return ("StringCol(length=%s, dflt=%r, shape=%s, pos=%s, indexed=%s)"
+                % (self.itemsize, self.dflt, self.shape, self._v_pos,
+                   self.indexed))
+
+
 class BoolCol(Col):
     "Defines a boolean column."
 
@@ -231,53 +323,6 @@ class BoolCol(Col):
 
 
 
-class StringCol(Col):
-    "Defines a string column."
-
-    def _setType(self, type_):
-        self.type       = records.CharType
-        self.stype      = str(records.CharType)
-
-    def _setDefault(self, dflt):
-        if dflt != None and not isinstance(dflt, str):
-            raise ValueError("Invalid default value: %s" % (dflt,))
-        self.dflt = dflt
-
-    def _setIndex(self, indexed):
-        super(Col, self)._setIndex(indexed)
-
-    def _setShape(self, shape):
-        super(Col, self)._setShape(shape)
-
-        # Set itemsize; forced to None to get it from the 'length' argument
-        self.itemsize = None
-
-    def __init__(self, length = None, dflt = None, shape = 1, pos = None,
-                 indexed = False):
-        Col.__init__(
-            self, dtype = 'CharType',
-            dflt = dflt, shape = shape, pos = pos, indexed = indexed)
-
-        # Some more work needed for constructor call idiosincracies:
-        # 'itemsize' is deduced from the default value if not specified.
-        if length is None and dflt:
-            length = len(dflt)  # 'dflt' has already been checked
-
-        if length is None:
-            raise ValueError("""\
-You must specify at least a length or a default value
-  where this length can be inferred from.""")
-
-        self.itemsize = length
-        self.recarrtype = revrecarrfmt[self.type]+str(self.itemsize)
-
-    def __repr__(self):
-        return ("StringCol(length=%s, dflt=%r, shape=%s, pos=%s, indexed=%s)"
-                % (self.itemsize, self.dflt, self.shape, self._v_pos,
-                   self.indexed))
-
-
-
 class IntCol(Col):
     "Defines an integer column."
 
@@ -826,12 +871,17 @@ type ``%s`` can not represent all concrete values in the enumeration"""
                 "name of default enumerated value is not a string: %r"
                 % (dflt,))
 
-        self.dflt = self.enum[dflt]
+        # The defaults are now numarray objects. However,
+        # for enumerated types, we still hold using python objects
+        # because some issues with Enum.__call__(NumArray) calls.
+        # F. Altet 2005-11-9
+        #self.dflt = self.enum[dflt]
+        self.dflt = NA.array(self.enum[dflt], type=self.type)
 
 
     def __repr__(self):
         return ('EnumCol(%s, %r, dtype=\'%s\', shape=%s, pos=%s, indexed=%s)'
-                % (self.enum, self.enum(self.dflt),
+                % (self.enum, self.enum(self.dflt[()]),
                    self.type, self.shape, self._v_pos, self.indexed))
 
 
@@ -940,7 +990,7 @@ class Description(object):
             newdict["_v_nestedlvl"] = nestedlvl + 1
 
         if "_v_byteorder" in keys and self._v_nestedlvl > 0:
-                raise KeyError, \
+            raise KeyError, \
 "You can only specify a byteorder in the root level of the description object, not in nested levels."
         # __check_validity__ must be check out prior to the keys loop
         if "__check_validity__" in keys:
@@ -1011,12 +1061,8 @@ class Description(object):
             if isinstance(object, Col):
                 newdict['_v_types'][k] = object.type
                 newdict['_v_stypes'][k] = object.stype
-                if hasattr(object, 'dflt') and not object.dflt is None:
-                    newdict['_v_dflts'][k] = object.dflt
-                else:
-                    newdict['_v_dflts'][k] = self._g_testtype(object)
+                newdict['_v_dflts'][k] = object.dflt
                 nestedFormats.append(str(object.shape) + object.recarrtype)
-                #nestedFormats.append(str(object.shape) + object.stype)
                 newdict['_v_shapes'][k] = object.shape
                 newdict['_v_itemsizes'][k] = object.itemsize
                 if isinstance(object.shape, tuple):
@@ -1042,11 +1088,9 @@ class Description(object):
         newdict['_v_nestedFormats'] = nestedFormats
         if self._v_nestedlvl == 0:
             # Get recursively nested _v_nestedNames and _v_nestedDescr attrs
-            self._g_getNestedNamesDescr()
+            self._g_setNestedNamesDescr()
             # Get pathnames for nested groups
-            self._g_getPathNames()
-            # Assign a name to the root container
-            newdict['_v_name'] = "_v_root"
+            self._g_setPathNames()
             # Assign the byteorder (if not yet)
             if not hasattr(self, "_v_byteorder"):
                 newdict["_v_byteorder"] = sys.byteorder
@@ -1083,30 +1127,7 @@ class Description(object):
         if pos1 > pos2:
             return 1
 
-    def _g_testtype(self, object):
-        """Test if datatype is valid and returns a default value for
-        each one.
-        """
-        datatype = object.type
-        if (isinstance(datatype, NA.IntegralType) or
-            isinstance(datatype, NA.FloatingType) or
-            isinstance(datatype, NA.ComplexType) or
-            isinstance(datatype, NA.BooleanType)):
-            dfltvalue = datatype.default
-        # Special cases
-        elif object.stype == 'Time32':  # integer time value
-            dfltvalue = int(0)
-        elif object.stype == 'Time64':  # floating point time value
-            dfltvalue = float(0)
-        # Add more code to check for validity on string type!
-        elif object.stype == "CharType":
-            dfltvalue = str("")
-        else:
-            raise TypeError, "DataType \'%s\' not supported!." \
-                  % datatype
-        return dfltvalue
-
-    def _g_getNestedNamesDescr(self):
+    def _g_setNestedNamesDescr(self):
         """Computes the nested names and descriptions for nested datatypes.
         """
         names = self._v_names
@@ -1117,35 +1138,85 @@ class Description(object):
             name = names[i]
             new_object = self._v_colObjects[name]
             if isinstance(new_object, Description):
-                new_object._g_getNestedNamesDescr()
+                new_object._g_setNestedNamesDescr()
                 # replace the column nested name by a correct tuple
                 self._v_nestedNames[i] = (name, new_object._v_nestedNames)
                 self._v_nestedDescr[i] = (name, new_object._v_nestedDescr)
 
-    def _g_getPathNames(self):
-        """Computes the pathnames for arbitrary nested descriptions.
+    def _g_setPathNames(self):
+        """Compute the pathnames for arbitrary nested descriptions.
+
+        This method sets the ``_v_pathname`` and ``_v_pathnames``
+        attributes of all the elements (both descriptions and columns)
+        in this nested description.
         """
-        # XXXX This should be reworked to use _v_walk iterator
-        stack = [self]
-        pathnames = self.__dict__["_v_pathnames"] = []  # global pathnames
-        self.__dict__["_v_pathname"] = ""
+
+        def getColsInOrder(description):
+            return [description._v_colObjects[colname]
+                    for colname in description._v_names]
+
+        def joinPaths(path1, path2):
+            if not path1:
+                return path2
+            return '%s/%s' % (path1, path2)
+
+        # The top of the stack always has a nested description
+        # and a list of its child columns
+        # (be they nested ``Description`` or non-nested ``Col`` objects).
+        # In the end, the list contains only a list of column paths
+        # under this one.
+        #
+        # For instance, given this top of the stack::
+        #
+        #   (<Description X>, [<Column A>, <Column B>])
+        #
+        # After computing the rest of the stack, the top is::
+        #
+        #   (<Description X>, ['a', 'a/m', 'a/n', ... , 'b', ...])
+
+        stack = []
+
+        # We start by pushing the top-level description
+        # and its child columns.
+        self._v_pathname = ''
+        stack.append((self, getColsInOrder(self)))
+
         while stack:
-            object = stack.pop()
-            names = object._v_names
-            for i in range(len(names)):
-                name = names[i]
-                new_object = object._v_colObjects[name]
-                pname = object._v_pathname
-                # pathnames for columns never start by "/"
-                if pname == "":
-                    new_object._v_pathname = name
-                else:
-                    new_object._v_pathname = pname + '/' + name
-                object._v_pathnames.append(new_object._v_pathname)
-                pathnames.append(new_object._v_pathname)
-                if isinstance(new_object, Description):
-                    new_object.__dict__["_v_pathnames"] = []
-                    stack.append(new_object)
+            desc, cols = stack.pop()
+            head = cols[0]
+
+            # What's the first child in the list?
+            if isinstance(head, Description):
+                # A nested description.  We remove it from the list and
+                # push it with its child columns.  This will be the next
+                # handled description.
+                head._v_pathname = joinPaths(desc._v_pathname, head._v_name)
+                stack.append((desc, cols[1:]))  # alter the top
+                stack.append((head, getColsInOrder(head)))  # new top
+            elif isinstance(head, Col):
+                # A non-nested column.  We simply remove it from the
+                # list and append its name to it.
+                head._v_pathname = joinPaths(desc._v_pathname, head._v_name)
+                cols.append(head._v_name)  # alter the top
+                stack.append((desc, cols[1:]))  # alter the top
+            else:
+                # Since paths and names are appended *to the end* of
+                # children lists, a string signals that no more children
+                # remain to be processed, so we are done with the
+                # description at the top of the stack.
+                assert isinstance(head, basestring)
+                # Assign the computed set of descendent column paths.
+                desc._v_pathnames = cols
+                if len(stack) > 0:
+                    # Compute the paths with respect to the parent node
+                    # (including the path of the current description)
+                    # and append them to its list.
+                    descName = desc._v_name
+                    colPaths = [joinPaths(descName, path) for path in cols]
+                    colPaths.insert(0, descName)
+                    parentCols = stack[-1][1]
+                    parentCols.extend(colPaths)
+                # (Nothing is pushed, we are done with this description.)
 
     def _v_walk(self, type="All"):
         """
