@@ -1,23 +1,18 @@
-########################################################################
-#
-#       License: BSD
-#       Created: February 11, 2005
-#       Author:  Ivan Vilata - reverse:com.carabos@ivilata
-#
-#       $Source$
-#       $Id: Node.py 1086 2005-07-18 17:08:57Z faltet $
-#
-########################################################################
-
 """
-Base class for PyTables nodes.
+Base class for PyTables nodes
+=============================
+
+:Author:   Ivan Vilata i Balaguer
+:Contact:  ivilata@carabos.com
+:Created:  2005-02-11
+:License:  BSD
+:Revision: $Id: Node.py 1597 2006-05-17 12:17:29Z ivilata $
+
 
 Classes:
 
 `Node`
     Abstract base class for all PyTables nodes.
-`HiddenNodeMixin`
-    Mixin class to create hidden nodes.
 
 Misc variables:
 
@@ -32,7 +27,9 @@ import warnings
 
 from tables.constants import MAX_TREE_DEPTH
 from tables.registry import classNameDict, classIdDict
-from tables.exceptions import NodeError, UndoRedoWarning, PerformanceWarning
+from tables.exceptions import \
+     ClosedNodeError, NodeError, UndoRedoWarning, PerformanceWarning
+from tables.utils import joinPath, splitPath, isVisiblePath
 from tables.undoredo import moveToShadow
 from tables.AttributeSet import AttributeSet
 
@@ -41,7 +38,7 @@ from tables.AttributeSet import AttributeSet
 __docformat__ = 'reStructuredText'
 """The format of documentation strings in this module."""
 
-__version__ = '$Revision: 1086 $'
+__version__ = '$Revision: 1597 $'
 """Repository version of this file."""
 
 
@@ -122,7 +119,7 @@ class Node(object):
     _v_hdf5name
         The name of this node in the hosting HDF5 file (a string).
     _v_pathname
-        The path name of this node in the tree (a string).
+        The path of this node in the tree (a string).
     _v_rootgroup
         The root group instance.  This is deprecated; please use
         ``node._v_file.root``.
@@ -152,6 +149,8 @@ class Node(object):
         Move or rename this node.
     _f_copy([newparent][, newname][, overwrite][, recursive][, **kwargs])
         Copy this node and return the new one.
+    _f_isVisible()
+        Is this node visible?
 
     Public methods (attribute handling):
 
@@ -176,6 +175,15 @@ class Node(object):
 
     # <properties>
 
+    # `_v_parent` is accessed via its file to avoid upwards references.
+    def _g_getparent(self):
+        (parentPath, nodeName) = splitPath(self._v_pathname)
+        return self._v_file._getNode(parentPath)
+
+    _v_parent = property(
+        _g_getparent, None, None, "The parent `Group` instance.")
+
+
     # '_v_rootgroup' is deprecated in favour of 'node._v_file.root'.
     def _g_getrootgroup(self):
         warnings.warn(DeprecationWarning("""\
@@ -218,12 +226,169 @@ class Node(object):
     # </properties>
 
 
-    def _g_setLocation(self, parent, ptname, h5name=None):
+    def __init__(self, parentNode, name, log=True):
+        # Remember to assign these values in the root group constructor
+        # if it does not use this method implementation!
+
+        self._v_file = None
+        """The hosting `File` instance."""
+        self._v_pathname = None
+        """The path of this node in the tree (a string)."""
+        self._v_name = None
+        """The name of this node in its parent group (a string)."""
+        self._v_hdf5name = None
+        """The name of this node in the hosting HDF5 file (a string)."""
+        self._v_depth = None
+        """The depth of this node in the tree (an non-negative integer value)."""
+        self._v__deleting = False
+        """Is the node being deleted?"""
+
+        self._v_objectID = None
+        """The identifier of this node in the hosting HDF5 file."""
+
+        validate = new = self._v_new  # set by subclass constructor
+
+        # Is the parent node a group?  Is it open?
+        self._g_checkGroup(parentNode)
+        parentNode._g_checkOpen()
+        file_ = parentNode._v_file
+
+        # Will the file be able to host a new node?
+        if new:
+            file_._checkWritable()
+
+        # Find out the matching HDF5 name.
+        ptname = name  # always the provided one
+        h5name = file_._h5NameFromPTName(ptname)
+
+        # Will creation be logged?
+        undoEnabled = file_.isUndoEnabled()
+        canUndoCreate = self._c_canUndoCreate
+        if undoEnabled and not canUndoCreate:
+            warnings.warn(
+                "creation can not be undone nor redone for this node",
+                UndoRedoWarning)
+
+        # Bind to the parent node and set location-dependent information.
+        if new:
+            # Only new nodes need to be referenced.
+            # Opened nodes are already known by their parent group.
+            parentNode._g_refNode(self, ptname, validate)
+        self._g_setLocation(parentNode, ptname, h5name)
+
+        try:
+            # hdf5Extension operations:
+            #   Update node attributes.
+            self._g_new(parentNode, h5name, init=True)
+            #   Create or open the node and get its object ID.
+            if new:
+                self._v_objectID = self._g_create()
+            else:
+                self._v_objectID = self._g_open()
+
+            # This allows extra operations after creating the node.
+            self._g_postInitHook()
+        except:
+            if new:
+                # Creation failed, this is no longer a child of the parent.
+                parentNode._g_unrefNode(ptname)
+            # If anything happens, the node must be closed
+            # to undo every possible registration made so far.
+            # We do *not* rely on ``__del__()`` doing it later,
+            # since it might never be called anyway.
+            self._f_close()
+            raise
+
+        # Finally, log creation of the node.
+        # This is made after the ``try`` because the node *has* been created!
+        if new and log and undoEnabled and canUndoCreate:
+            file_._log('CREATE', self._v_pathname)
+
+
+    def __del__(self):
+        # Closed `Node` instances can not be killed and revived.
+        # Instead, accessing a closed and deleted (from memory, not
+        # disk) one yields a *new*, open `Node` instance.  This is
+        # because of two reasons:
+        #
+        # 1. Predictability.  After closing a `Node` and deleting it,
+        #    only one thing can happen when accessing it again: a new,
+        #    open `Node` instance is returned.  If closed nodes could be
+        #    revived, one could get either a closed or an open `Node`.
+        #
+        # 2. Ease of use.  If the user wanted to access a closed node
+        #    again, the only condition would be that no references to
+        #    the `Node` instance were left.  If closed nodes could be
+        #    revived, the user would also need to force the closed
+        #    `Node` out of memory, which is not a trivial task.
+        #
+        if not self._f_isOpen():
+            return
+
+        # If we get here, the `Node` is still open.
+        file_ = self._v_file
+        if self._v_pathname in file_._aliveNodes:
+            # If the node is alive, kill it (to save it).
+            file_._killNode(self)
+        else:
+            # The node is already dead and there are no references to it,
+            # so follow the usual deletion procedure.
+            # This means closing the (still open) node.
+            # `self._v__deleting` is asserted so that the node
+            # does not try to unreference itself again from the file.
+            self._v__deleting = True
+            self._f_close()
+
+
+    def _g_preKillHook(self):
+        """Code to be called before killing the node."""
+        pass
+
+
+    def _g_postReviveHook(self):
+        """Code to be called after reviving the node."""
+        pass
+
+
+    def _g_create(self):
+        """Create a new HDF5 node and return its object identifier."""
+        raise NotImplementedError
+
+
+    def _g_open(self):
+        """Open an existing HDF5 node and return its object identifier."""
+        raise NotImplementedError
+
+
+    def _f_isOpen(self):
+        """Is this node open?"""
+
+        if not '_v_file' in self.__dict__:
+            return False
+        # When the construction of a node is aborted because of an exception,
+        # the ``_v_file`` attribute might exist but be set to `None`,
+        # so the node is still considered closed.
+        return self._v_file is not None
+
+
+    def _g_checkOpen(self):
+        """
+        Check that the node is open.
+
+        If the node is closed, a `ClosedNodeError` is raised.
+        """
+
+        if not self._f_isOpen():
+            raise ClosedNodeError("the node object is closed")
+        assert self._v_file.isopen, "found an open node in a closed file"
+
+
+    def _g_setLocation(self, parentNode, ptname, h5name=None):
         """
         Set location-dependent attributes.
 
         Sets the location-dependent attributes of this node to reflect
-        that it is placed under the specified `parent` node, with the
+        that it is placed under the specified `parentNode`, with the
         specified PyTables and HDF5 names (`ptname` and `h5name`,
         respectively).  If the HDF5 name is ``None``, it is found using
         the translation map from the parent's file.
@@ -233,20 +398,18 @@ class Node(object):
         `PerformanceWarning` is issued.
         """
 
-        file_ = parent._v_file
-        parentDepth = parent._v_depth
+        file_ = parentNode._v_file
+        parentDepth = parentNode._v_depth
 
-        mydict = self.__dict__
-        mydict['_v_file'] = file_
+        self._v_file = file_
+        self._v_pathname = joinPath(parentNode._v_pathname, ptname)
 
         if h5name is None:
-            h5name = file_.trMap.get(ptname, ptname)
-        mydict['_v_name'] = ptname
-        mydict['_v_hdf5name'] = h5name
+            h5name = file_._h5NameFromPTName(ptname)
+        self._v_name = ptname
+        self._v_hdf5name = h5name
 
-        mydict['_v_parent'] = parent
-        mydict['_v_pathname'] = parent._g_join(ptname)
-        mydict['_v_depth'] = parentDepth + 1
+        self._v_depth = parentDepth + 1
 
         # Check if the node is too deep in the tree.
         if parentDepth >= MAX_TREE_DEPTH:
@@ -259,13 +422,13 @@ be ready to see PyTables asking for *lots* of memory and possibly slow I/O"""
         file_._refNode(self, self._v_pathname)
 
 
-    def _g_updateLocation(self):
+    def _g_updateLocation(self, newParentPath):
         """
         Update location-dependent attributes.
 
         Updates location data when an ancestor node has changed its
-        location in the hierarchy.  In fact, this method is expected to
-        be called by an ancestor of this node.
+        location in the hierarchy to `newParentPath`.  In fact, this
+        method is expected to be called by an ancestor of this node.
 
         This also triggers the update of file references to this node.
         If the maximum recommended node depth is exceeded, a
@@ -273,15 +436,12 @@ be ready to see PyTables asking for *lots* of memory and possibly slow I/O"""
         unique.
         """
 
-        file_ = self._v_file
-        parent = self._v_parent
-        parentDepth = parent._v_depth
-        oldPathname = self._v_pathname
-        newPathname = parent._g_join(self._v_name)
+        oldPath = self._v_pathname
+        newPath = joinPath(newParentPath, self._v_name)
+        parentDepth = newParentPath.count('/')
 
-        mydict = self.__dict__
-        mydict['_v_pathname'] = newPathname
-        mydict['_v_depth'] = parentDepth + 1
+        self._v_pathname = newPath
+        self._v_depth = parentDepth + 1
 
         # Check if the node is too deep in the tree.
         if parentDepth >= MAX_TREE_DEPTH:
@@ -290,8 +450,12 @@ moved descendent node is exceeding the recommended maximum depth (%d);\
 be ready to see PyTables asking for *lots* of memory and possibly slow I/O"""
                           % (MAX_TREE_DEPTH,), PerformanceWarning)
 
-        file_._unrefNode(oldPathname)
-        file_._refNode(self, newPathname)
+        file_ = self._v_file
+        file_._unrefNode(oldPath)
+        file_._refNode(self, newPath)
+
+        # Tell dependent objects about the new location of this node.
+        self._g_updateDependent()
 
 
     def _g_delLocation(self):
@@ -304,57 +468,69 @@ be ready to see PyTables asking for *lots* of memory and possibly slow I/O"""
         file_ = self._v_file
         pathname = self._v_pathname
 
-        mydict = self.__dict__
-        mydict['_v_file'] = None
+        self._v_file = None
+        self._v_pathname = None
+        self._v_name = None
+        self._v_hdf5name = None
+        self._v_depth = None
 
-        mydict['_v_name'] = None
-        mydict['_v_hdf5name'] = None
-
-        mydict['_v_parent'] = None
-        mydict['_v_pathname'] = None
-        mydict['_v_depth'] = None
-
-        file_._unrefNode(pathname)
+        # If the node object is being deleted,
+        # it has already been unreferenced from the file.
+        if not self._v__deleting:
+            file_._unrefNode(pathname)
 
 
-    def _g_afterOpen(self):
-        """
-        Tidy object tree after its creation.
-
-        Put here everything that you want to arrange *after* the object
-        tree has been created.
-
-        Warning!  This hook will be called only for visible nodes, not
-        hidden nodes.  However, you can call the tidy code of hidden
-        nodes from the visible ones.
-        """
-        # The default is to do nothing. Overload this in appropriate Nodes.
+    def _g_postInitHook(self):
+        """Code to be run after node creation and before creation logging."""
         pass
 
 
+    def _g_updateDependent(self):
+        """
+        Update dependent objects after a location change.
+
+        All dependent objects (but not nodes!) referencing this node
+        must be updated here.
+        """
+        if '_v_attrs' in self.__dict__:
+            self._v_attrs._g_updateNodeLocation(self)
+
+
     def _f_close(self):
         """
         Close this node in the tree.
 
-        This makes the node inaccessible from the object tree.  The
-        closing operation is *not* recursive, i.e. closing a group does
-        not close its children.  On nodes with data, it may flush it to
-        disk.
+        This releases all resources held by the node, so it should not
+        be used again.  On nodes with data, it may be flushed to disk.
+
+        The closing operation is *not* recursive, i.e. closing a group
+        does not close its children.
         """
 
-        # Close and remove the associated AttributeSet
-        # only if it has already been placed in the object's dictionary.
-        mydict = self.__dict__
-        if '_v_attrs' in mydict:
-            self._v_attrs._f_close()
-            del mydict['_v_attrs']
+        # After calling ``_f_close()``, two conditions are met:
+        #
+        #   1. The node object is detached from the tree.
+        #   2. *Every* attribute of the node is removed.
+        #
+        # Thus, cleanup operations used in ``_f_close()`` in sub-classes
+        # must be run *before* calling the method in the superclass.
 
-        # Remove references from and to the parent group.
-        parent = self._v_parent
-        if isinstance(parent, classNameDict['Group']):
-            parent._g_unrefNode(self._v_name)
+        if not self._f_isOpen():
+            return  # the node is already closed
+
+        myDict = self.__dict__
+
+        # Close the associated `AttributeSet`
+        # only if it has already been placed in the object's dictionary.
+        if '_v_attrs' in myDict:
+            self._v_attrs._f_close()
+
+        # Detach the node from the tree if necessary.
         self._g_delLocation()
 
+        # Finally, clear all remaining attributes from the object.
+        myDict.clear()
+
 
     def _g_remove(self, recursive):
         """
@@ -367,7 +543,13 @@ be ready to see PyTables asking for *lots* of memory and possibly slow I/O"""
         It does not log the change.
         """
 
-        raise NotImplementedError
+        # Remove the node from the PyTables hierarchy.
+        self._v_parent._g_unrefNode(self._v_name)
+        # Close the node itself.
+        self._f_close()
+        # hdf5Extension operations:
+        #   Remove the node from the HDF5 hierarchy.
+        self._g_delete()
 
 
     def _f_remove(self, recursive=False):
@@ -379,7 +561,9 @@ be ready to see PyTables asking for *lots* of memory and possibly slow I/O"""
         raised.
         """
 
+        self._g_checkOpen()
         file_ = self._v_file
+        file_._checkWritable()
 
         if file_.isUndoEnabled():
             if self._c_canUndoMove:
@@ -422,11 +606,15 @@ be ready to see PyTables asking for *lots* of memory and possibly slow I/O"""
 
         # hdf5Extension operations:
         #   Update node attributes.
-        self._g_new(newParent, self._v_hdf5name)
+        self._g_new(newParent, self._v_hdf5name, init=False)
         #   Move the node.
-        self._v_parent._g_moveNode(oldPathname, self._v_pathname)
-        #   Update attribute set attributes.
-        self._v_attrs._g_new(self)
+        #self._v_parent._g_moveNode(oldPathname, self._v_pathname)
+        self._v_parent._g_moveNode(oldParent._v_objectID, oldName,
+                                   newParent._v_objectID, newName,
+                                   oldPathname, self._v_pathname)
+
+        # Tell dependent objects about the new location of this node.
+        self._g_updateDependent()
 
 
     def _f_rename(self, newname):
@@ -462,6 +650,7 @@ be ready to see PyTables asking for *lots* of memory and possibly slow I/O"""
         node in place.
         """
 
+        self._g_checkOpen()
         file_ = self._v_file
         oldParent = self._v_parent
         oldName = self._v_name
@@ -479,6 +668,9 @@ you should specify at least a ``newparent`` or a ``newname`` parameter""")
         newparent = file_.getNode(newparent)  # Does the new parent exist?
         self._g_checkGroup(newparent)  # Is it a group?
 
+        # The movement always fails if the hosting file can not be modified.
+        file_._checkWritable()
+
         if newparent._v_file is not file_:  # Is it in the same file?
             raise NodeError("""\
 nodes can not be moved across databases; please make a copy of the node""")
@@ -511,7 +703,7 @@ nodes can not be moved across databases; please make a copy of the node""")
             file_._log('MOVE', oldPathname, newPathname)
 
 
-    def _g_copy(self, newParent, newName, recursive, **kwargs):
+    def _g_copy(self, newParent, newName, recursive, log, **kwargs):
         """
         Copy this node and return the new one.
 
@@ -522,7 +714,7 @@ nodes can not be moved across databases; please make a copy of the node""")
         ignored.  On recursive copies, all keyword arguments must be
         passed on to the children invocation of this method.
 
-        It does not log the change.
+        If `log` is true, the change is logged.
         """
         raise NotImplementedError
 
@@ -559,65 +751,66 @@ nodes can not be moved across databases; please make a copy of the node""")
         the same group.
         """
 
-        file_ = self._v_file
-        oldParent = self._v_parent
-        oldName = self._v_name
+        self._g_checkOpen()
+        srcFile = self._v_file
+        srcParent = self._v_parent
+        srcName = self._v_name
+
+        dstParent = newparent
+        dstName = newname
 
         # Set default arguments.
-        if newparent is None and newname is None:
+        if dstParent is None and dstName is None:
             raise NodeError("""\
 you should specify at least a ``newparent`` or a ``newname`` parameter""")
-        if newparent is None:
-            newparent = oldParent
-        if newname is None:
-            newname = oldName
+        if dstParent is None:
+            dstParent = srcParent
+        if dstName is None:
+            dstName = srcName
 
         # Validity checks on arguments.
-        newparent = file_.getNode(newparent)  # Does the new parent exist?
-        self._g_checkGroup(newparent)  # Is it a group?
+        # If ``dstParent`` is a path, it *must* be in the source file!
+        dstParent = srcFile.getNode(dstParent)  # Does the new parent exist?
+        self._g_checkGroup(dstParent)  # Is it a group?
 
         dolog = True  # Is it in the same file?
-        if newparent._v_file is not file_ and file_.isUndoEnabled():
+        if dstParent._v_file is not srcFile and srcFile.isUndoEnabled():
             warnings.warn("""\
 copying across databases can not be undone nor redone from this database""",
                           UndoRedoWarning)
             dolog = False
 
         # Copying over itself?
-        if (newparent is oldParent) and (newname == oldName):
+        if (dstParent is srcParent) and (dstName == srcName):
             raise NodeError(
                 "source and destination nodes are the same node: ``%s``"
                 % (self._v_pathname,))
 
         if recursive:
-            self._g_checkNotContains(newparent)  # Copying into itself?
+            self._g_checkNotContains(dstParent)  # Copying into itself?
         self._g_maybeRemove(  # Copying over an existing node?
-            newparent, newname, overwrite)
-
-        undoEnabled = file_.isUndoEnabled()
-        canUndoCreate = self._c_canUndoCreate
-        if undoEnabled and not canUndoCreate:
-            warnings.warn(
-                "copying can not be undone nor redone for this node",
-                UndoRedoWarning)
+            dstParent, dstName, overwrite)
 
         # Copy the node.
-        newNode = self._g_copy(newparent, newname, recursive, **kwargs)
+        # The constructor of the new node takes care of logging.
+        return self._g_copy(dstParent, dstName, recursive, True, **kwargs)
 
-        # Log the change.
-        if dolog and undoEnabled and canUndoCreate:
-            file_._log('CREATE', newNode._v_pathname)
 
-        return newNode
+    def _f_isVisible(self):
+        """Is this node visible?"""
+        self._g_checkOpen()
+        return isVisiblePath(self._v_pathname)
 
 
     def _g_checkGroup(self, node):
         # Node must be defined in order to define a Group.
         # However, we need to know Group here.
         # Using classNameDict avoids a circular import.
+        if not isinstance(node, classNameDict['Node']):
+            raise TypeError("new parent is not a node: %r" % (node,))
         if not isinstance(node, classNameDict['Group']):
-            raise TypeError(
-                "new parent node is not a group: %r" % (node,))
+            raise TypeError("new parent node ``%s`` is not a group"
+                            % node._v_pathname)
 
 
     def _g_checkNotContains(self, node):
@@ -634,67 +827,19 @@ copying across databases can not be undone nor redone from this database""",
                 raise NodeError("""\
 destination group ``%s`` already has a node named ``%s``; \
 you may want to use the ``overwrite`` argument""" % (parent._v_pathname, name))
-            getattr(parent, name)._f_remove(True)
+            parent._f_getChild(name)._f_remove(True)
 
 
     def _g_isDescendentOf(self, group):
-        file_ = self._v_file
-
-        # If the nodes are in different files its is not worth
-        # climibing up the hierarchy.
-        if file_ is not group._v_file:
+        # The nodes are in different files.
+        if self._v_file is not group._v_file:
             return False
 
-        parent = self._v_parent
-        while parent is not file_:
-            if parent is group:
-                return True
-            parent = parent._v_parent
-        return False
-
-
-    def _g_putUnder(self, parent, name):
-        """
-        Put node in hierarchy.
-
-        Puts this node (whether newly created or already existing in an
-        HDF5 file) in the hierarchy, under the specified `parent` group,
-        with the specified `name`.  Please note that the given `name`
-        has a different meaning depending on when the node was created.
-
-        If the node has been created in this session, the provided
-        `name` is assumed to be its PyTables name, and its HDF5 name is
-        found using the translation map from the parent's file.  Else,
-        the provided `name` is assumed to be its HDF5 name, and its
-        PyTables name is found using the translation map from the
-        parent's file.
-        """
-
-        # All this will eventually end up in the node constructor.
-
-        validate = new = self._v_new
-
-        # Find out the PyTables and HDF5 names.
-        trMap = parent._v_file.trMap
-        if new:
-            # New node: get HDF5 name from PyTables name.
-            ptname = name  # always the provided one
-            h5name = trMap.get(name, name)
-        else:
-            # Opened node: get PyTables name from HDF5 name.
-            h5name = ptname = name  # always the provided one
-            # This code might seem inefficient but it will be rarely used.
-            for (ptname_, h5name_) in trMap.iteritems():
-                if h5name_ == h5name:
-                    ptname = ptname_
-                    break
-
-        parent._g_refNode(self, ptname, validate)
-        self._g_setLocation(parent, ptname, h5name)
-
-        # hdf5Extension operations:
-        #   Update node attributes.
-        self._g_new(parent, h5name)
+        # This check avoids walking up the tree.
+        prefix = group._v_pathname + '/'
+        if prefix == '//':
+            return True  # all nodes descend from the root group
+        return self._v_pathname.startswith(prefix)
 
 
     def _g_checkName(self, name):
@@ -744,22 +889,6 @@ you may want to use the ``overwrite`` argument""" % (parent._v_pathname, name))
 
 
 
-class HiddenNodeMixin:
-
-    """
-    Mixin class to create hidden nodes.
-
-    Node classes having this class added to their superclasses will have
-    their instance nodes rendered invisible to all kinds of node listing
-    operations such as `File.walkNodes()` or `Group._f_listNodes()`.
-    Nonetheless, they will still be accessible via `File.getNode()`,
-    ``getattr()`` and natural naming.
-    """
-
-    pass
-
-
-
 ## Local Variables:
 ## mode: python
 ## py-indent-offset: 4
