@@ -7,8 +7,7 @@
 #       Created: November 12, 2003
 #       Author:  Francesc Altet - faltet@carabos.com
 #
-#       $Source: /home/ivan/_/programari/pytables/svn/cvs/pytables/pytables/tables/VLArray.py,v $
-#       $Id: VLArray.py 1153 2005-09-06 11:32:10Z faltet $
+#       $Id: VLArray.py 1570 2006-05-02 12:58:22Z faltet $
 #
 ########################################################################
 
@@ -43,14 +42,20 @@ try:
 except ImportError:
     Numeric_imported = False
 
+try:
+    import numpy
+    numpy_imported = True
+except ImportError:
+    numpy_imported = False
+
 import tables.hdf5Extension as hdf5Extension
-from tables.utils import processRangeRead, convertIntoNA, convToFlavor
+from tables.utils import processRangeRead, convertToNA, convToFlavor, idx2long
 from tables.Atom import Atom, ObjectAtom, VLStringAtom, StringAtom, EnumAtom
 from tables.Leaf import Leaf
 
 
 
-__version__ = "$Revision: 1153 $"
+__version__ = "$Revision: 1570 $"
 
 # default version for VLARRAY objects
 #obversion = "1.0"    # initial version
@@ -128,8 +133,17 @@ class VLArray(hdf5Extension.VLArray, Leaf):
     # </undo-redo support>
 
 
-    def __init__(self, atom=None, title = "",
-                 filters = None, expectedsizeinMB = 1.0):
+    # <properties>
+    shape = property(
+        lambda self: (self.nrows,), None, None,
+        "The shape of the stored array.")
+    # </properties>
+
+
+    def __init__(self, parentNode, name,
+                 atom=None, title="",
+                 filters=None, expectedsizeinMB=1.0,
+                 log=True):
         """Create the instance Array.
 
         Keyword arguments:
@@ -151,23 +165,61 @@ class VLArray(hdf5Extension.VLArray, Leaf):
             time and the amount of memory used.
 
         """
+
+        self._v_version = None
+        """The object version of this array."""
+
+        self._v_new = new = atom is not None
+        """Is this the first time the node has been created?"""
         self._v_new_title = title
+        """New title for this node."""
         self._v_new_filters = filters
+        """New filter properties for this array."""
         self._v_expectedsizeinMB = expectedsizeinMB
-        self._v_maxTuples = 100    # Maybe enough for most applications
-        # Check if we have to create a new object or read their contents
-        # from disk
-        if atom is not None:
-            self.atom = atom
-            self._v_new = 1
-        else:
-            self._v_new = 0
+        """The expected size of the array in MiB."""
 
-    def _create(self):
+        self._v_maxTuples = 100       # maybe enough for most applications
+        """The maximum number of rows that are read on each chunk iterator."""
+        self._v_chunksize = None
+        """The HDF5 chunk size for ``VLArray`` objects."""
+
+        # Miscellaneous iteration rubbish.
+        self._start = None
+        """Starting row for the current iteration."""
+        self._stop = None
+        """Stopping row for the current iteration."""
+        self._step = None
+        """Step size for the current iteration."""
+        self._nrowsread = None
+        """Number of rows read up to the current state of iteration."""
+        self._startb = None
+        """Starting row for current buffer."""
+        self._stopb = None
+        """Stopping row for current buffer. """
+        self._row = None
+        """Current row in iterators (sentinel)."""
+        self._init = False
+        """Whether we are in the middle of an iteration or not (sentinel)."""
+        self.listarr = None
+        """Current buffer in iterators."""
+
+        # Documented (*public*) attributes.
+        self.atom = atom
+        """
+        An `Atom` instance representing the shape, type and flavor of
+        the atomic objects to be saved.
+        """
+        self.nrow = None
+        """On iterators, this is the index of the current row."""
+        self.nrows = None
+        """The total number of rows."""
+
+        super(VLArray, self).__init__(parentNode, name, new, filters, log)
+
+
+    def _g_create(self):
         """Create a variable length array (ragged array)."""
 
-        global obversion
-
         self._v_version = obversion
         # Only support for creating objects in system byteorder
         self.byteorder  = sys.byteorder
@@ -192,8 +244,36 @@ class VLArray(hdf5Extension.VLArray, Leaf):
         self._v_chunksize = calcChunkSize(self._v_expectedsizeinMB,
                                           self.filters.complevel)
         self.nrows = 0     # No rows in creation time
-        self.shape = (0,)
-        self._createArray(self._v_new_title)
+
+        self._v_objectID = self._createArray(self._v_new_title)
+        return self._v_objectID
+
+
+    def _g_open(self):
+        """Get the metadata info for an array in file."""
+
+        (self._v_objectID, self.nrows) = self._openArray()
+
+        flavor = self.flavor
+        stype = self._atomicstype
+        # First, check the special cases VLString and Object types
+        if flavor == "VLString":
+            self.atom = VLStringAtom()
+        elif flavor == "Object":
+            self.atom = ObjectAtom()
+        elif stype == 'CharType':
+            self.atom = StringAtom(self._atomicshape, self._basesize,
+                                   flavor, warn=False)
+        elif stype == 'Enum':
+            (enum, type_) = self._loadEnum()
+            self.atom = EnumAtom(enum, type_, self._atomicshape,
+                                 flavor, warn=False)
+            self._atomictype = type_
+        else:
+            self.atom = Atom(stype, self._atomicshape, flavor, warn=False)
+
+        return self._v_objectID
+
 
     def _checkShape(self, naarr):
         # Check for zero dimensionality array
@@ -313,7 +393,8 @@ please put them in a single sequence object"""),
         if self.atom.flavor == "Object":
             # Special case for a generic object
             # (to be pickled and saved as an array of unsigned bytes)
-            object = numarray.array(cPickle.dumps(object), type=numarray.UInt8)
+            object = numarray.array(cPickle.dumps(object, 0),
+                                    type=numarray.UInt8)
         elif self.atom.flavor == "VLString":
             # Special case for a generic object
             # (to be pickled and saved as an array of unsigned bytes)
@@ -322,16 +403,15 @@ please put them in a single sequence object"""),
 """The object "%s" is not of type String or Unicode.""" % (str(object))
             try:
                 object = object.encode('utf-8')
-            except UnicodeError:
-                (typerr, value, traceback) = sys.exc_info()
-                raise ValueError, "Problems when converting the object '%s' to the encoding 'utf-8'. The error was: %s" % (object, value)
+            except UnicodeError, ue:
+                raise ValueError, "Problems when converting the object '%s' to the encoding 'utf-8'. The error was: %s" % (object, ue)
             object = numarray.array(object, type=numarray.UInt8)
 
         if len(object) > 0:
             # The object needs to be copied to make the operation safe
             # to in-place conversion.
             copy = self._atomicstype in ['Time64']
-            naarr = convertIntoNA(object, self.atom, copy)
+            naarr = convertToNA(object, self.atom, copy)
             nobjects = self._checkShape(naarr)
         else:
             nobjects = 0
@@ -340,31 +420,8 @@ please put them in a single sequence object"""),
         self._append(naarr, nobjects)
 
         self.nrows += 1
-        self.shape = (self.nrows,)
 
 
-    def _open(self):
-        """Get the metadata info for an array in file."""
-
-        self.nrows = self._openArray()
-        self.shape = (self.nrows,)
-
-        flavor = self.flavor
-        stype = self._atomicstype
-        # First, check the special cases VLString and Object types
-        if flavor == "VLString":
-            self.atom = VLStringAtom()
-        elif flavor == "Object":
-            self.atom = ObjectAtom()
-        elif stype == 'CharType':
-            self.atom = StringAtom(self._atomicshape, self._basesize, flavor)
-        elif stype == 'Enum':
-            (enum, type_) = self._loadEnum()
-            self.atom = EnumAtom(enum, type_, self._atomicshape, flavor)
-            self._atomictype = type_
-        else:
-            self.atom = Atom(stype, self._atomicshape, flavor)
-
     def iterrows(self, start=None, stop=None, step=None):
         """Iterate over all the rows or a range.
 
@@ -378,7 +435,7 @@ please put them in a single sequence object"""),
     def __iter__(self):
         """Iterate over all the rows."""
 
-        if not hasattr(self, "_init"):
+        if not self._init:
             # If the iterator is called directly, assign default variables
             self._start = 0
             self._stop = self.nrows
@@ -393,13 +450,13 @@ please put them in a single sequence object"""),
         self._nrowsread = self._start
         self._startb = self._start
         self._row = -1   # Sentinel
-        self._init = 1    # Sentinel
+        self._init = True  # Sentinel
         self.nrow = self._start - self._step    # row number
 
     def next(self):
         "next() method for __iter__() that is called on each iteration"
         if self._nrowsread >= self._stop:
-            del self._init
+            self._init = False
             raise StopIteration        # end of iteration
         else:
             # Read a chunk of rows
@@ -424,7 +481,8 @@ please put them in a single sequence object"""),
 
         """
 
-        if type(key) in (int,long):
+        if (type(key) in (int,long) or
+            (numpy_imported and isinstance(key, numpy.integer))):
             if key >= self.nrows:
                 raise IndexError, "Index out of range"
             if key < 0:
@@ -472,7 +530,8 @@ please put them in a single sequence object"""),
             raise IndexError, "You cannot specify more than two dimensions"
         nrow, rng = keys
         # Process the first index
-        if type(nrow) not in (int,long):
+        if not (type(nrow) in (int,long) or
+                (numpy_imported and isinstance(nrow, numpy.integer))):
             raise IndexError, "The first dimension only can be an integer"
         if nrow >= self.nrows:
             raise IndexError, "First index out of range"
@@ -480,7 +539,8 @@ please put them in a single sequence object"""),
             # To support negative values
             nrow += self.nrows
         # Process the second index
-        if type(rng) in (int,long):
+        if (type(rng) in (int,long) or
+            (numpy_imported and isinstance(rng, numpy.integer))):
             start = rng; stop = start+1; step = 1
         elif isinstance(rng, slice):
             start, stop, step = rng.start, rng.stop, rng.step
@@ -494,7 +554,8 @@ please put them in a single sequence object"""),
         if self.atom.flavor == "Object":
             # Special case for a generic object
             # (to be pickled and saved as an array of unsigned bytes)
-            object = numarray.array(cPickle.dumps(object), type=numarray.UInt8)
+            object = numarray.array(cPickle.dumps(object, 0),
+                                    type=numarray.UInt8)
         elif self.atom.flavor == "VLString":
             # Special case for a generic object
             # (to be pickled and saved as an array of unsigned bytes)
@@ -503,18 +564,15 @@ please put them in a single sequence object"""),
 """The object "%s" is not of type String or Unicode.""" % (str(object))
             try:
                 object = object.encode('utf-8')
-            except UnicodeError:
-                (typerr, value, traceback) = sys.exc_info()
-                raise ValueError, "Problems when converting the object '%s' to the encoding 'utf-8'. The error was: %s" % (object, value)
+            except UnicodeError, ue:
+                raise ValueError, "Problems when converting the object '%s' to the encoding 'utf-8'. The error was: %s" % (object, ue)
             object = numarray.array(object, type=numarray.UInt8)
 
-        value = convertIntoNA(object, self.atom)
+        value = convertToNA(object, self.atom)
         nobjects = self._checkShape(value)
 
-        #nobjects = len(value)
-
         # Get the previous value
-        #naarr = self.read(nrow, nrow+1, 1, slice_specified=0)
+        nrow = idx2long(nrow)   # To convert any possible numpy scalar value
         naarr = self._readArray(nrow, nrow+1, 1)[0]
         nobjects = len(naarr)
         if len(value) > nobjects:
@@ -522,13 +580,16 @@ please put them in a single sequence object"""),
 "Length of value (%s) is larger than number of elements in row (%s)" % \
 (len(value), nobjects)
         # Assign the value to it
+        # The values can be numpy scalars. Convert them before building the slice.
+        if start is not None: start = idx2long(start)
+        if stop is not None: stop = idx2long(stop)
+        if step is not None: step = idx2long(step)
         try:
             naarr[slice(start, stop, step)] = value
-        except:  #XXX
-            (typerr, value2, traceback) = sys.exc_info()
+        except Exception, exc:  #XXX
             raise ValueError, \
 "Value parameter:\n'%r'\ncannot be converted into an array object compliant vlarray[%s] row: \n'%r'\nThe error was: <%s>" % \
-        (value, keys, naarr[slice(start, stop, step)], value2)
+        (value, keys, naarr[slice(start, stop, step)], exc)
 
         if naarr.size() > 0:
             self._modify(nrow, naarr, nobjects)
@@ -542,7 +603,7 @@ please put them in a single sequence object"""),
             listarr = []
         else:
             listarr = self._readArray(start, stop, step)
-        if self.flavor <> "NumArray":
+        if self.flavor <> "numarray":
             # Convert the list to the right flavor
             outlistarr = [convToFlavor(self, arr, "VLArray")
                           for arr in listarr ]
@@ -553,12 +614,13 @@ please put them in a single sequence object"""),
             outlistarr = listarr
         return outlistarr
 
-    def _g_copyWithStats(self, group, name, start, stop, step, title, filters):
+    def _g_copyWithStats(self, group, name, start, stop, step,
+                         title, filters, log):
         "Private part of Leaf.copy() for each kind of leaf"
         # Build the new VLArray object
-        object = self._v_file.createVLArray(
+        object = VLArray(
             group, name, self.atom, title=title, filters=filters,
-            expectedsizeinMB=self._v_expectedsizeinMB, _log = False)
+            expectedsizeinMB=self._v_expectedsizeinMB, log=log)
         # Now, fill the new vlarray with values from the old one
         # This is not buffered because we cannot forsee the length
         # of each record. So, the safest would be a copy row by row.
@@ -581,7 +643,6 @@ please put them in a single sequence object"""),
             nbytes += nobjects*atomsize
             nrowscopied +=1
         object.nrows = nrowscopied
-        object.shape = (nrowscopied,)
         return (object, nbytes)
 
     def __repr__(self):
