@@ -4,8 +4,7 @@
 #       Created: October 1, 2002
 #       Author:  Francesc Altet - faltet@carabos.com
 #
-#       $Source: /home/ivan/_/programari/pytables/svn/cvs/pytables/pytables/tables/__init__.py,v $
-#       $Id: __init__.py 1015 2005-06-17 17:55:14Z faltet $
+#       $Id: __init__.py 1262 2005-09-30 14:56:04Z ivilata $
 #
 ########################################################################
 
@@ -25,16 +24,13 @@ Misc variables:
 
     __version__
     hdf5Version
-    extVersion
 
 """
 
 # Necessary imports to get versions stored on the Pyrex extension
-from tables.utilsExtension import \
-     getPyTablesVersion, getExtVersion, getHDF5Version
+from tables.utilsExtension import getPyTablesVersion, getHDF5Version
 
 __version__ = getPyTablesVersion()
-extVersion  = getExtVersion()
 hdf5Version = getHDF5Version()
 
 from tables.utilsExtension import isHDF5  # deprecated
@@ -66,6 +62,7 @@ from tables.UnImplemented import UnImplemented
 __all__ = [
     # Exceptions and warnings:
     'HDF5ExtError',
+    'ClosedNodeError', 'ClosedFileError', 'FileModeError',
     'NaturalNameWarning', 'NodeError', 'NoSuchNodeError',
     'UndoRedoError', 'UndoRedoWarning',
     'PerformanceWarning',
