@@ -18,6 +18,7 @@ arrays.
 
 import sys
 import types
+import re
 
 import numarray
 import numarray.strings
@@ -25,13 +26,36 @@ import numarray.records
 
 from tables.attributeaccess import AttributeAccess
 import nriterators
+import utils
 
+try:
+    import numpy
+    numpy_imported = True
+except ImportError:
+    numpy_imported = False
 
 __docformat__ = 'reStructuredText'
 """The format of documentation strings in this module."""
 
 
 
+# formats regular expression
+# allows multidimension spec with a tuple syntax in front of the letter code
+# '(2,3)f4' and ' (  2 ,  3  )  f4  ' are equally allowed
+format_re = re.compile(r'(?P<repeat> *[(]?[ ,0-9]*[)]? *)(?P<dtype>[A-Za-z0-9.]*)'
+)
+
+cbyteorder = { 'little': '<', 'big': '>' }
+
+revfmt = {'Int8':'i1',  'UInt8':'u1',
+          'Int16':'i2', 'UInt16':'u2',
+          'Int32':'i4', 'UInt32':'u4',
+          'Int64':'i8', 'UInt64':'u8',
+          'Float32':'f4', 'Float64':'f8',
+          'Complex32':'c8', 'Complex64':'c16',
+          'Bool':'b1'}
+
+
 def _isThereStructure(formats, descr, buffer):
     """
     Check if buffer structure is given.  It must be given in order to
@@ -180,7 +204,7 @@ def _checkBufferStructure(structure, buffer):
                 raise TypeError("""field descriptors must be strings""")
 
 
-def _matchFormats(seq1, seq2):
+def _matchFormats_orig(seq1, seq2):
     """Check if two flat formats lists are equivalent."""
 
     # Lists must have the same length
@@ -196,6 +220,33 @@ def _matchFormats(seq1, seq2):
             raise ValueError("""buffer formats don't match those """
                 """provided by the format specification""")
 
+# This should be quite faster than _matchFormats_orig.
+# F. Altet 2006-01-18
+def _matchFormats(seq1, seq2):
+    """Check if two flat formats lists are equivalent."""
+
+    # Lists must have the same length
+    if len(seq1) != len(seq2):
+        raise ValueError("""buffer structure doesn't match that """
+            """provided by the format specification""")
+
+    # Elements in the same position must describe the same format
+    for (f1, f2) in zip(seq1, seq2):
+        (repeat1, dtype1) = format_re.match(f1.strip()).groups()
+        (repeat2, dtype2) = format_re.match(f2.strip()).groups()
+        dtype1 = dtype1.strip()
+        dtype2 = dtype2.strip()
+        if dtype1 in revfmt: dtype1 = revfmt[dtype1]
+        if dtype2 in revfmt: dtype2 = revfmt[dtype2]
+        if repeat1 in ['','1']: dtype1 = '1'+dtype1
+        if repeat2 in ['','1']: dtype2 = '1'+dtype2
+        if dtype1 != dtype2:
+            raise ValueError("""buffer formats don't match those """
+                """provided by the format specification""")
+
+def _renameFields(recarray, newFieldNames):
+    """Rename the fields of a recarray"""
+    pass  # To be completed
 
 def _narrowRecArray(recarray, startField, newFieldNames):
     """
@@ -215,11 +266,11 @@ def _narrowRecArray(recarray, startField, newFieldNames):
         recarray._data, recarray._formats[iStartField:iEndField],
         shape=recarray._shape, names=newFieldNames, byteoffset=byteOffset,
         bytestride=recarray._bytestride, byteorder=recarray._byteorder,
-        aligned=recarray.isaligned())
+        aligned=recarray._rec_aligned)
 
 
 def array(buffer=None, formats=None, shape=0, names=None,
-        byteorder=sys.byteorder, aligned=0, descr=None):
+          byteorder=sys.byteorder, aligned=0, descr=None):
     """
     Create a new instance of a `NestedRecArray`.
 
@@ -327,8 +378,8 @@ def array(buffer=None, formats=None, shape=0, names=None,
     ...     descr=[('id', 'Int64'), ('pos', '(2,)Float32'),
     ...            ('info', [('name', 'a2'), ('value', 'Complex64')])])
 
-    Note how `formats` and `descr` mimic the structure of each the
-    elements in `buffer`.
+    Note how `formats` and `descr` mimic the structure of each element
+    in `buffer`.
 
     Example 2
     ---------
@@ -347,26 +398,103 @@ def array(buffer=None, formats=None, shape=0, names=None,
     and `aligned` arguments.
     """
 
-    # Check if a buffer structure is given. It must be given in order to
-    # disambiguate possible ambiguities
-    _isThereStructure(formats, descr, buffer)
-
     # Check if buffer structure is specified using descr OR formats (and,
     # optionally, names)
     _onlyOneSyntax(descr, formats, names)
 
+    # Create or check the descr format
+    if descr is None:
+        if formats is not None:
+            descr = makeDescr(formats, names)
+        # `buffer` can still be some object which describes its own structure,
+        # so `descr`/`formats` are not yet required.
+    else:
+        _checkDescr(descr)
+        _checkFieldsInDescr(descr)
+    # After this, if it exists a formats, it will always exists a descr
+
     # This is to keep compatibility with numarray.records.array function
     if isinstance(formats, str):
         formats = formats.split(',')
     if isinstance(names, str):
         names = names.split(',')
 
-    # Check the descr format
-    # Check for '/' in descr
-    if descr is None:
-        descr = makeDescr(formats, names)
-    _checkDescr(descr)
-    _checkFieldsInDescr(descr)
+    # First, check for easily convertible objects (NRA, NA and NumPy
+    # objects)
+    # F. Altet 2006-01-20
+    if isinstance(buffer, NestedRecArray):
+        buffer = buffer.copy()    # Always return a copy of the data
+        # Return as soon as possible is not descr, formats or names specified
+        if (descr is None and formats is None and names is None):
+            return buffer
+        # Check that descriptions are consistent
+        if descr is not None:
+            fmts = [item for item in nriterators.flattenFormats(makeFormats(descr))]
+            # Check just the formats, not the names
+            if _matchFormats(fmts, buffer._formats):
+                raise ValueError, \
+"""buffer structure doesn't match that provided by the format
+    specification."""
+            # New description is compatible. Assign it to the NRA
+            buffer.descr = descr
+            # Also the names (they may have been changed in new description)
+            if names is None:
+                names = makeNames(descr)
+            # Assignements in both the NRA and flatArray are necessary
+            buffer._names =  [i for i in nriterators.flattenNames(names)]
+            buffer._flatArray._names =  [i for i in nriterators.flattenNames(names)]
+        return buffer
+
+    if isinstance(buffer, numarray.records.RecArray):
+        buffer = buffer.copy()    # Always return a copy of the data
+        # Return as soon as possible is not descr, formats or names specified
+        if (descr is None and formats is None and names is None):
+            descr = makeDescr(buffer._formats, buffer._names)
+            # Return the nested recarray
+            return NestedRecArray(buffer, descr)
+
+        # Check that descriptions are consistent
+        if formats is not None and _matchFormats(formats, buffer._formats):
+            raise ValueError, \
+"""buffer structure doesn't match that provided by the format
+    specification."""
+        if names is not None:
+            buffer._names = names
+        # Also, the names may have been changed in new description
+        elif descr is not None:
+            buffer._names = makeNames(descr)
+        if descr is None:
+            descr = makeDescr(buffer._formats, buffer._names)
+        # Refresh the cache of fields (just in case the names has changed)
+        buffer._fields = buffer._get_fields()
+        # Return the nested recarray
+        return NestedRecArray(buffer, descr)
+
+    # Check for numpy ndarrays, recarrays, records or scalar records
+    if (numpy_imported and (isinstance(buffer, numpy.ndarray) or
+                            type(buffer) == numpy.rec.recarray or
+                            type(buffer) == numpy.rec.record or
+                            type(buffer) == numpy.void)):
+        # Try to convert into a nestedrecarray
+        try:
+            nra = utils.fromnumpy(buffer, copy=True)
+        except Exception, exc:  #XXX
+            raise ValueError, \
+"""buffer parameter of type numpy cannot be converted into a NestedRecArray
+object. The error was: <%s>""" % (exc,)
+
+        # Check that descriptions are consistent
+        if descr is not None:
+            fmt1 = [i for i in nriterators.flattenFormats(makeFormats(nra.descr))]
+            fmt2 = [i for i in nriterators.flattenFormats(makeFormats(descr))]
+            if _matchFormats(fmt1, fmt2):
+                raise ValueError, \
+"""buffer structure doesn't match that provided by the format specification."""
+        return nra
+
+    # Check if a buffer structure is given. It must be given in order to
+    # disambiguate possible ambiguities
+    _isThereStructure(formats, descr, buffer)
 
     # Check the formats format
     if formats is None:
@@ -383,12 +511,8 @@ def array(buffer=None, formats=None, shape=0, names=None,
     flatNames = [item for item in nriterators.flattenNames(names)]
 
     # Check the buffer structure (order matters!)
-    if isinstance(buffer, NestedRecArray):
-        _matchFormats( flatFormats, buffer._flatArray._formats)
-    elif isinstance(buffer, numarray.records.RecArray):
-        _matchFormats( flatFormats, buffer._formats)
-    elif (isinstance(buffer, types.ListType) or
-          isinstance(buffer, types.TupleType)):
+    if (isinstance(buffer, types.ListType) or
+        isinstance(buffer, types.TupleType)):
         if (isinstance(buffer[0], numarray.NumArray) or
             isinstance(buffer[0], numarray.strings.CharArray)):
             return fromarrays(buffer, formats=formats,
@@ -397,16 +521,6 @@ def array(buffer=None, formats=None, shape=0, names=None,
     elif buffer:
         _checkBufferStructure( descr, buffer)
 
-    # If buffer is a RecArray then we have almost done with the creation
-    if isinstance(buffer, numarray.records.RecArray):
-        if isinstance(buffer, NestedRecArray):
-            flatBuffer = buffer.asRecArray()
-        else:
-            flatBuffer = buffer.copy()
-        newFlatArray = _narrowRecArray(flatBuffer, flatBuffer._names[0],
-            flatNames)
-        return NestedRecArray(newFlatArray, descr)
-
     # Flatten the buffer (if any)
     if buffer is None:
         flatBuffer = None
@@ -439,7 +553,7 @@ def _checkArrayList(arrayList):
 def fromarrays(arrayList, formats=None, names=None, shape=0,
                byteorder=sys.byteorder, aligned=0, descr=None):
     """
-    Create a new instance of a `NestedRecArray` from field arrays.
+    Create a new instance of a `NestedRecArray` from `arrayList` arrays.
 
     This function can be used to build a new array of nested records
     from a list of arrays, one for each field.  The new array is
@@ -517,13 +631,114 @@ def fromarrays(arrayList, formats=None, names=None, shape=0,
     nriterators.flattenArraysList(arrayList, descr, flatArrayList)
     ra = numarray.records.fromarrays(flatArrayList, formats=flatFormats,
         names=flatNames, shape=shape, byteorder=byteorder, aligned=aligned)
-    
+
     # Create the nested recarray
     nra = NestedRecArray(ra, descr)
 
     return nra
 
 
+def convertToAPDescr(descr, byteorder):
+    """
+    Convert a NRA `descr` descriptor into a true array protocol description.
+    """
+
+    # Build a descriptor compliant with array protocol
+    # (http://numeric.scipy.org/array_interface.html)
+    i = nriterators.getIter(descr)
+    if not i:
+        return
+
+    try:
+        item = i.next()
+        while item:
+            if isinstance(item[1], str):
+                # parse the formats into repeats and formats
+                try:
+                    (_repeat, _dtype) = format_re.match(item[1].strip()).groups()
+                except TypeError, AttributeError:
+                    raise ValueError('format %s is not recognized' %  _fmt[i])
+                _dtype = _dtype.strip()
+                # CharType needs special treatment
+                if _dtype[0] == 'a':
+                    _dtype = '|S'+_dtype[1:]
+                else:
+                    if _dtype in revfmt:
+                        # _dtype is in long format
+                        _dtype = cbyteorder[byteorder]+revfmt[_dtype]
+                    elif _dtype in revfmt.values():
+                        # _dtype is already in short format
+                        _dtype = cbyteorder[byteorder]+_dtype
+                    else:
+                        # This should never happen
+                        raise ValueError, \
+                              "Fatal error: format %s not recognized." % (_dtype)
+                # Return the column
+                if _repeat in ['','1']:
+                    # scalar case
+                    yield (item[0], _dtype)
+                else:
+                    yield (item[0], _dtype, eval(_repeat))
+            else:
+                l = []
+                for j in convertToAPDescr(item[1], byteorder):
+                    l.append(j)
+                yield (item[0], l)
+            item = i.next()
+    except StopIteration:
+        pass
+
+
+def convertFromAPDescr(array_descr):
+    """
+    Convert a true description of the array protocol in one suited for NRA.
+    """
+
+    # Get an iterator for the array descrition
+    i = nriterators.getIter(array_descr)
+    if not i:
+        return
+
+    try:
+        item = i.next()
+        while item:
+            if isinstance(item[1], str):
+                _dtype = item[1].strip()
+                _dtype = _dtype[1:]  # remove the byteorder
+                if _dtype in revfmt.values():
+                    _dtype = _dtype
+                elif _dtype[0] == 'S':
+                    # CharType needs special treatment
+                    _dtype = 'a'+_dtype[1:]
+                elif _dtype[0] == "V":
+                    raise NotImplementedError, """ \
+Padding fields are not supported yet. Try to provide objects without padding fields.
+"""
+                elif _dtype[0] == "U":
+                    raise NotImplementedError, """ \
+Unicode fields are not supported yet. Try to provide objects without unicode fields.
+"""
+                else:
+                    # All the other fields are not supported
+                    raise ValueError, \
+                          "Fatal error: format %s not supported." % (_dtype)
+                # Return the column
+                if len(item) <= 2:
+                    # scalar case
+                    yield (item[0], _dtype)
+                else:
+                    shape = item[2]
+                    yield (item[0], "%s%s" % (shape, _dtype))
+            else:
+                l = []
+                for j in convertFromAPDescr(item[1]):
+                    l.append(j)
+                yield (item[0], l)
+            item = i.next()
+    except StopIteration:
+        pass
+
+
 class NestedRecArray(numarray.records.RecArray):
 
     """
@@ -563,15 +778,18 @@ class NestedRecArray(numarray.records.RecArray):
             recarray._data, recarray._formats, shape=recarray._shape,
             names=recarray._names, byteoffset=recarray._byteoffset,
             bytestride=recarray._bytestride,
-            byteorder=recarray._byteorder, aligned=recarray.isaligned())
+            byteorder=recarray._byteorder, aligned=recarray._rec_aligned)
         # ``_strides`` is not properly copied from the original array,
         # so, the copy must be made by hand. :[
         self._strides = recarray._strides
 
         self._flatArray = recarray
         self.descr = descr
+        # Create a true compliant array protocol description
+        self.array_descr = [d for d in convertToAPDescr(descr, self._byteorder)]
+
+        self.fields = AttributeAccess(self, 'field')  # XXXX
 
-        self.fields = AttributeAccess(self, 'field')
         """
         Provides attribute access to fields.
 
@@ -635,8 +853,6 @@ shape=%(rsShape)s)''' % rsData
         if isinstance(key, slice) and not isinstance(value, _RecArray):
             # Conversion of the value to an array will need a little help
             # until structure inference is supported.
-            print 'Value', value
-            print 'Description', self.descr
             value = array(value, descr=self.descr)
         #super(NestedRecArray, self).__setitem__(key, value)
         # Call the setitem method with the flatArray mate instead.
@@ -660,7 +876,7 @@ shape=%(rsShape)s)''' % rsData
 
     def __add__(self, other):
         """Add two NestedRecArray objects in a row wise manner."""
-    
+
         if isinstance(other, NestedRecArray):
             return NestedRecArray(self._flatArray + other._flatArray,
                 self.descr)
